from insights.tests import context_wrap
from insights.parsers.rhn_schema_stats import DBStatsLog

POSTGRESQL_LOG = """
 schema |             table              |   rows
--------+--------------------------------+----------
 public | rhnsnapshotpackage             | 47428950
 public | rhnpackagefile                 | 32174333
 public | rhnpackagecapability           | 12934215
 public | rhnpackagechangelogrec         | 11269933
 public | rhnchecksum                    | 10129746
 public | rhnactionconfigrevision        |  2894957
 public | rhnpackageprovides             |  2712442
 public | rhnpackagerequires             |  2532861
 public | rhn_command_target             |  1009152
 public | rhnconfigfilename              |        0
 public | rhnxccdfidentsystem            |        0
 public | rhndistchannelmap              |        0
 public | rhnactionvirtshutdown          |        0
 public | rhnpublicchannelfamily         |        0
(402 rows)

                        name                        | type |             table              |                                                                                               src
----------------------------------------------------+------+--------------------------------+-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 chkpb_probe_type_ck                                | c    | rhn_check_probe                | ((probe_type)::text = 'check'::text)
 cmdtg_target_type_ck                               | c    | rhn_command_target             | ((target_type)::text = ANY (ARRAY[('cluster')))
 contperm_wbuserid_fk                               | f    | web_user_contact_permission    |
 log_id_pk                                          | p    | log                            |
 log_user_id_fk                                     | f    | log                            |
 personal_info_web_user_id_fk                       | f    | web_user_personal_info         |
 prbst_probe_id_scout_id_pk                         | p    | rhn_probe_state                |
 pxt_sessions_pk                                    | p    | pxtsessions                    |
 pxtsessions_user                                   | f    | pxtsessions                    |
 qrtz_blob_triggers_pkey                            | p    | qrtz_blob_triggers             |
 rhn_actchainent_aid_fk                             | f    | rhnactionchainentry            |
 rhn_actchainent_sid_fk                             | f    | rhnactionchainentry            |
 rhn_action_archived_ck                             | c    | rhnaction                      | (archived = ANY (ARRAY[(0)::numeric, (1)::numeric]))
 rhn_action_at_fk                                   | f    | rhnaction                      |
 rhn_action_chain_id_pk                             | p    | rhnactionchain                 |
 rhn_action_oid_fk                                  | f    | rhnaction                      |
 rhn_action_pk                                      | p    | rhnaction                      |
 rhn_action_prereq_fk                               | f    | rhnaction                      |
 rhn_avsp_aid_fk                                    | f    | rhnactionvirtschedulepoller    |
 rhn_avsp_aid_pk                                    | p    | rhnactionvirtschedulepoller    |
 rhn_avstart_aid_pk                                 | p    | rhnactionvirtstart             |
 wusi_ipb_ck                                        | c    | web_user_site_info             | (is_po_box = ANY (ARRAY['1'::bpchar, '0'::bpchar]))
 wusi_type_fk                                       | f    | web_user_site_info             |
 wusi_wuid_fk                                       | f    | web_user_site_info             |
 wust_type_pk                                       | p    | web_user_site_type             |
(1771 rows)

                label                |  created   |  modified  |       name       | epoch | version  | release
-------------------------------------+------------+------------+------------------+-------+----------+----------
 schema-from-20130309-133209         | 2013-03-09 | 2013-07-02 | satellite-schema |       | 5.5.0.17 | 1.el6sat
 schema-from-20130702-090610         | 2013-07-02 | 2014-05-19 | satellite-schema |       | 5.5.0.20 | 1.el6sat
 schema-migrate-from-20140519-111223 | 2014-05-19 | 2014-12-08 | satellite-schema |       | 5.5.0.22 | 1.el6sat
 schema-from-20141208-181050         | 2014-12-08 | 2014-12-09 | satellite-schema |       | 5.6.0.10 | 1.el6sat
 schema-from-20150803-160946         | 2015-08-03 | 2015-10-16 | satellite-schema |       | 5.6.0.26 | 1.el6sat
 schema-from-20151016-113838         | 2015-10-16 | 2015-10-16 | satellite-schema |       | 5.7.0.11 | 1.el6sat
 schema                              | 2015-10-16 | 2015-10-16 | satellite-schema |       | 5.7.0.19 | 1.el6sat
(11 rows)
""".strip()

ORACLE_LOG = """
RHN_REDIRECT_TYPES: 4
RHN_SATELLITE_STATE: 0
RHN_SAT_CLUSTER: 0
RHN_SAT_CLUSTER_PROBE: 0
RHN_SAT_NODE: 0
RHN_SCHEDULES: 1
RHN_SCHEDULE_DAYS: 7
RHN_SCHEDULE_TYPES: 3
RHN_SCHEDULE_WEEKS: 0
RHN_SEMANTIC_DATA_TYPE: 7
RHN_SERVER_MONITORING_INFO: 0
RHN_SERVICE_PROBE_ORIGINS: 0
RHN_SNMP_ALERT: 0
RHN_STRATEGIES: 6
RHN_THRESHOLD_TYPE: 4
RHN_TIME_ZONE_NAMES: 22
RHNERRATASEVERITY: 112

PL/SQL procedure successfully completed.


CONSTRAINT NAME \t       TYPE TABLE NAME\t\t\t   SEARCH CONDITION\t\t\t\t\t\t\t\t    REFERENCED CONSTRAINT
------------------------------ ---- ------------------------------ -------------------------------------------------------------------------------- ------------------------------
SYS_C003263\t\t       C    RHNERRATASEVERITY\t\t   "LABEL" IS NOT NULL
RHN_ERRATATMP_ADV_TYPE_CK      C    RHNERRATATMP\t\t   advisory_type in ('Bug Fix Advisory',
\t\t\t\t\t\t\t\t   \t\t\t\t\t\t\t    'Product E

SYS_C003988\t\t       C    RHNERRATATMP\t\t   "LAST_MODIFIED" IS NOT NULL
WUSI_IPB_CK\t\t       C    WEB_USER_SITE_INFO\t\t   is_po_box in ('1','0')
WUSI_ID_PK\t\t       P    WEB_USER_SITE_INFO
WUSI_TYPE_FK\t\t       R    WEB_USER_SITE_INFO\t\t\t\t\t\t\t\t\t\t\t\t    WUST_TYPE_PK
WUSI_WUID_FK\t\t       R    WEB_USER_SITE_INFO\t\t\t\t\t\t\t\t\t\t\t\t    WEB_CONTACT_PK

2613 rows selected.


LABEL\t\t\t       CREATED\t  MODIFIED   NAME\t\t       EPOCH\t  VERSION    RELEASE
------------------------------ ---------- ---------- ------------------------- ---------- ---------- ----------
schema-from-20131111-143638    2013-11-11 2013-11-11 satellite-schema\t\t\t  5.5.0.13   1.el5sat
schema\t\t\t       2014-05-01 2014-05-01 satellite-schema\t\t\t  5.5.0.22   1.el5sat

8 rows selected.
""".strip()

NO_LOG = """
"""


def test_rhn_schema_stats_pgs():

    pg_log = DBStatsLog(context_wrap(POSTGRESQL_LOG))
    assert 'rhnsnapshotPACKAGE' in pg_log
    assert 'rhnsnapshotpackage_' not in pg_log
    assert pg_log.get_table('chkpb_probe_type_ck') == []
    assert pg_log.get_table('RHN_COMMAND_TARGET', ic=False) == []
    assert pg_log.get_table('RHN_COMMAND_TARGET') == [
        {'rows': '1009152', 'schema': 'public', 'table': 'rhn_command_target'},
        {'name': 'cmdtg_target_type_ck',
         'src': "((target_type)::text = ANY (ARRAY[('cluster')))",
         'table': 'rhn_command_target', 'type': 'c'}
    ]


def test_rhn_schema_stats_ora():
    ora_log = DBStatsLog(context_wrap(ORACLE_LOG))
    assert 'rhnerratatmP' in ora_log
    assert 'rhnerratatmP_' not in ora_log
    assert ora_log.get_table('rhnerrataseverity', ic=False) == []
    assert ora_log.get_table('rhnerrataseverity') == [
        {'rows': 112, 'table': 'RHNERRATASEVERITY'},
        {'CONSTRAINT NAME': 'SYS_C003263', 'REFERENCED CONSTRAINT': '',
         'SEARCH CONDITION': '"LABEL" IS NOT NULL',
         'TABLE NAME': 'RHNERRATASEVERITY', 'TYPE': 'C'}
    ]


def test_rhn_schema_stats_none():
    log = DBStatsLog(context_wrap(NO_LOG))
    assert list(log.data.keys()) == []



"""
LsVarCachePulp - command ``ls -lan /var/cache/pulp``
====================================================

The ``ls -lan /var/cache/pulp`` command provides information for the listing of the ``/var/cache/pulp`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Sample directory list::

    total 0
    drwxrwxr-x.  5 48 1000 216 Jan 21 12:56 .
    drwxr-xr-x. 10  0    0 121 Jan 20 13:57 ..
    lrwxrwxrwx.  1  0    0  19 Jan 21 12:56 cache -> /var/lib/pulp/cache
    drwxr-xr-x.  2 48   48   6 Jan 21 13:03 reserved_resource_worker-0@dhcp130-202.gsslab.pnq2.redhat.com
    drwxr-xr-x.  2 48   48   6 Jan 21 02:03 reserved_resource_worker-1@dhcp130-202.gsslab.pnq2.redhat.com
    drwxr-xr-x.  2 48   48   6 Jan 20 14:03 resource_manager@dhcp130-202.gsslab.pnq2.redhat.com

Examples:

    >>> "journal" in ls_var_cache_pulp
    False
    >>> "/var/cache/pulp" in ls_var_cache_pulp
    True
"""


from insights.specs import Specs

from insights import CommandParser, parser, FileListing


@parser(Specs.ls_var_cache_pulp)
class LsVarCachePulp(CommandParser, FileListing):
    """Parses output of ``ls -lan /var/cache/pulp`` command."""
    pass



from .. import XMLParser, parser
from insights.specs import Specs


@parser(Specs.rhn_entitlement_cert_xml)
class RHNCertConf(XMLParser):
    """Class to parse the xml files ``rhn-entitlement-cert.xml*``

    Attributes:
        data (dict): A dict likes
        {
            'product': 'RHN-SATELLITE-001'
            'satellite-version': '5.2'
            'signature': '-----BEGIN PGP SIGNATURE-----....'
            'channel-families':
            {
                'rhel-cluster': {'quantity':'10'}
                'sam-rhel-server-6': {'quantity':'102', 'flex':'0'}
                ...
            }
            ...
        }
        ---
        And there may be patterns of "rhn_entitlement_cert.xml" files on the host,
        you can use the 'file_name' attribute to check where the settings are
        actually gotten from. E.g:
        ---
            rhn_certs = shared[rhn_cert]
            for cert in rhn_certs:
                if cert.file_name == 'rhn_entitlement_cert.xml':
                   cf = cert.get('channel_families')
                   ...

    ---Sample---
        <?xml version="1.0" encoding="UTF-8"?>
        <rhn-cert version="0.1">
          <rhn-cert-field name="product">RHN-SATELLITE-001</rhn-cert-field>
          <rhn-cert-field name="owner">Clay's Precious Satellite</rhn-cert-field>
          <rhn-cert-field name="issued">2005-01-11 00:00:00</rhn-cert-field>
          <rhn-cert-field name="expires">2005-03-11 00:00:00</rhn-cert-field>
          <rhn-cert-field name="slots">30</rhn-cert-field>
          <rhn-cert-field name="provisioning-slots">30</rhn-cert-field>
          <rhn-cert-field name="nonlinux-slots">30</rhn-cert-field>
          <rhn-cert-field name="channel-families" quantity="10" family="rhel-cluster"/>
          <rhn-cert-field name="channel-families" quantity="30" family="rhel-ws-extras"/>
          <rhn-cert-field name="channel-families" quantity="10" family="rhel-gfs"/>
          <rhn-cert-field name="channel-families" quantity="10" family="rhel-es-extras"/>
          <rhn-cert-field name="channel-families" quantity="40" family="rhel-as"/>
          <rhn-cert-field name="channel-families" quantity="30" family="rhn-tools"/>
          <rhn-cert-field name="channel-families" quantity="102" flex="0" family="sam-rhel-server-6"/>
          <rhn-cert-field name="channel-families" quantity="102" flex="51" family="cf-tools-5-beta"/>
          <rhn-cert-field name="satellite-version">5.2</rhn-cert-field>
          <rhn-cert-field name="generation">2</rhn-cert-field>
          <rhn-cert-signature>
        -----BEGIN PGP SIGNATURE-----
        Version: Crypt::OpenPGP 1.03

        iQBGBAARAwAGBQJCAG7yAAoJEJ5yna8GlHkysOkAn07qmlUrkGKs7/5yb8H/nboG
        mhHkAJ9wdmqOeKfcBa3IUDL53oNMEBP/dg==
        =0Kv7
        -----END PGP SIGNATURE-----
        </rhn-cert-signature>
        </rhn-cert>
    """

    def parse_dom(self):

        rhn_cert = {}
        # ignore empty xml file

        channel_familes = {}
        for field in self.dom.findall(".//rhn-cert-field"):
            family = field.get('family')
            if family:
                channel_familes[family] = dict(
                    (k, v) for k, v in field.items() if k not in ('name', 'family'))
            elif field.text:
                rhn_cert[field.get('name')] = field.text
        # for all channel families
        rhn_cert['channel-families'] = channel_familes
        singature = self.dom.findall(".//rhn-cert-signature")
        rhn_cert['signature'] = singature[0].text

        return rhn_cert



from insights.tests import context_wrap
from insights.parsers.crypto_policies import CryptoPoliciesBind
from insights.parsers import SkipException
import pytest

CONFIG = """
disable-algorithms "." {
RSAMD5;
DSA;
};
disable-ds-digests "." {
GOST;
};
""".strip()

CONFIG_EMPTY_SECTIONS = """
disable-algorithms "." {
};
disable-ds-digests "." {
};
""".strip()


def test_crypto_policies_bind():
    result = CryptoPoliciesBind(context_wrap(CONFIG))
    assert "GOST" in result.disable_ds_digests
    assert "DSA" in result.disable_algorithms
    assert ["RSAMD5", "DSA"] == result.disable_algorithms
    assert ["GOST"] == result.disable_ds_digests


def test_crypto_policies_bind_empty():
    with pytest.raises(SkipException):
        CryptoPoliciesBind(context_wrap(""))
    result = CryptoPoliciesBind(context_wrap(CONFIG_EMPTY_SECTIONS))
    assert [] == result.disable_algorithms
    assert [] == result.disable_ds_digests



import doctest
import pytest
from insights.parsers import sys_vmbus, SkipException
from insights.tests import context_wrap


BLANK = """
""".strip()

NO_RESULT = """
 Id    Name                           State
----------------------------------------------------
""".strip()

DEVICE_ID_1 = """
{47505500-0001-0000-3130-444531444234}
""".strip()

CLASS_ID_1 = """
{44c4f61d-4444-4400-9d52-802e27ede19f}
""".strip()

CLASS_ID_2 = """
{3375baf4-9e15-4b30-b765-67acb10d607b}
""".strip()

CLASS_ID_3 = """
{da0a7802-e377-4aac-8e77-0558eb1073f8}
""".strip()


def test_device_id():
    output = sys_vmbus.SysVmbusDeviceID(context_wrap(DEVICE_ID_1, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/device_id'))
    assert output.file_name == 'device_id'
    assert output.id == '47505500-0001-0000-3130-444531444234'


def test_class_id():
    output = sys_vmbus.SysVmbusClassID(context_wrap(CLASS_ID_1, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/class_id'))
    assert output.file_name == 'class_id'
    assert output.id == '44c4f61d-4444-4400-9d52-802e27ede19f'
    assert output.desc == 'PCI Express pass-through'

    output = sys_vmbus.SysVmbusClassID(context_wrap(CLASS_ID_2, path='/sys/bus/vmbus/devices/4487b255-b88c-403f-bb51-d1f69cf17f87/class_id'))
    assert output.file_name == 'class_id'
    assert output.id == '3375baf4-9e15-4b30-b765-67acb10d607b'
    assert output.desc == 'Unknown'

    output = sys_vmbus.SysVmbusClassID(context_wrap(CLASS_ID_3, path='/sys/bus/vmbus/devices/5620e0c7-8062-4dce-aeb7-520c7ef76171/class_id'))
    assert output.file_name == 'class_id'
    assert output.id == 'da0a7802-e377-4aac-8e77-0558eb1073f8'
    assert output.desc == 'Synthetic framebuffer adapter'


def test_blank_output():
    with pytest.raises(SkipException):
        output = sys_vmbus.SysVmbusDeviceID(context_wrap(BLANK, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/device_id'))
        assert output is None

    with pytest.raises(SkipException):
        output = sys_vmbus.SysVmbusClassID(context_wrap(BLANK, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/class_id'))
        assert output is None


def test_documentation():
    failed_count, tests = doctest.testmod(
        sys_vmbus,
        globs={
            'vmbus_device': sys_vmbus.SysVmbusDeviceID(context_wrap(DEVICE_ID_1, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/device_id')),
            'vmbus_class': sys_vmbus.SysVmbusClassID(context_wrap(CLASS_ID_1, path='/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531444234/class_id'))
        }
    )
    assert failed_count == 0



"""
PartedL - command ``parted -l``
===============================

This module provides processing for the ``parted`` command.  The output is parsed
by the ``PartedL`` class.  Attributes are provided for each field for the disk,
and a list of ``Partition`` class objects, one for each partition in the output.

Typical content of the ``parted -l`` command output
looks like::

    Model: ATA TOSHIBA MG04ACA4 (scsi)
    Disk /dev/sda: 4001GB
    Sector size (logical/physical): 512B/512B
    Partition Table: gpt
    Disk Flags: pmbr_boot

    Number  Start   End     Size    File system  Name  Flags
     1      1049kB  2097kB  1049kB                     bios_grub
     2      2097kB  526MB   524MB   xfs
     3      526MB   4001GB  4000GB                     lvm

The columns may vary depending upon the type of device.

Note:
    The examples in this module may be executed with the following command:

    ``python -m insights.parsers.parted``

Examples:
    >>> parted_data = '''
    ... Model: ATA TOSHIBA MG04ACA4 (scsi)
    ... Disk /dev/sda: 4001GB
    ... Sector size (logical/physical): 512B/512B
    ... Partition Table: gpt
    ... Disk Flags: pmbr_boot
    ...
    ... Number  Start   End     Size    File system  Name  Flags
    ...  1      1049kB  2097kB  1049kB                     bios_grub
    ...  2      2097kB  526MB   524MB   xfs
    ...  3      526MB   4001GB  4000GB                     lvm
    ... '''.strip()
    >>> from insights.tests import context_wrap
    >>> shared = {PartedL: PartedL(context_wrap(parted_data))}
    >>> parted_info = shared[PartedL]
    >>> parted_info.data
    {'partition_table': 'gpt', 'sector_size': '512B/512B', 'disk_flags': 'pmbr_boot', 'partitions': [{'end': '2097kB', 'name': 'bios_grub', 'number': '1', 'start': '1049kB', 'flags': 'bios_grub', 'file_system': 'bios_grub', 'size': '1049kB'}, {'start': '2097kB', 'size': '524MB', 'end': '526MB', 'number': '2', 'file_system': 'xfs'}, {'end': '4001GB', 'name': 'lvm', 'number': '3', 'start': '526MB', 'flags': 'lvm', 'file_system': 'lvm', 'size': '4000GB'}], 'model': 'ATA TOSHIBA MG04ACA4 (scsi)', 'disk': '/dev/sda', 'size': '4001GB'}
    >>> parted_info.data['model']
    'ATA TOSHIBA MG04ACA4 (scsi)'
    >>> parted_info.disk
    '/dev/sda'
    >>> parted_info.logical_sector_size
    '512B'
    >>> parted_info.physical_sector_size
    '512B'
    >>> parted_info.boot_partition
    >>> parted_info.data['disk_flags']
    'pmbr_boot'
    >>> len(parted_info.partitions)
    3
    >>> parted_info.partitions[0].data
    {'end': '2097kB', 'name': 'bios_grub', 'number': '1', 'start': '1049kB', 'flags': 'bios_grub', 'file_system': 'bios_grub', 'size': '1049kB'}
    >>> parted_info.partitions[0].number
    '1'
    >>> parted_info.partitions[0].start
    '1049kB'
    >>> parted_info.partitions[0].end
    '2097kB'
    >>> parted_info.partitions[0].size
    '1049kB'
    >>> parted_info.partitions[0].file_system
    'bios_grub'
    >>> parted_info.partitions[0].type
    >>> parted_info.partitions[0].flags
    'bios_grub'
"""
from .. import parser, CommandParser
from ..parsers import ParseException, parse_fixed_table
from insights.specs import Specs


class Partition(object):
    """Class to contain information for one partition.

    Represents the values from one row of the partition information from the
    ``parted`` command.  Column names have been converted to lowercase and are
    provided as attributes.  Column names may vary so the ``get`` method may
    be used to check for the presence of a column.

    Attributes:
        data (dict): Dictionary of partition information keyed by column names
            in lowercase.
    """
    def __init__(self, data):
        self.data = data

    @property
    def number(self):
        """str: Partition number."""
        return self.data.get('number')

    @property
    def start(self):
        """str: Starting location for the partition."""
        return self.data.get('start')

    @property
    def end(self):
        """str: Ending location for the partition."""
        return self.data.get('end')

    @property
    def size(self):
        """str: Size of the partition."""
        return self.data.get('size')

    @property
    def file_system(self):
        """str: File system type."""
        return self.data.get('file_system')

    @property
    def type(self):
        """str: File system type."""
        return self.data.get('type')

    @property
    def flags(self):
        """str: Partition flags."""
        return self.data.get('flags')

    def get(self, item):
        """Get information for column ``item`` or ``None`` if not present."""
        return self.data.get(item)


@parser(Specs.parted__l)
class PartedL(CommandParser):
    """Class to represent attributes of the ``parted`` command output.

    The columns may vary depending upon the type of device.

    Attributes:
        data (dict): Dictionary of information returned by ``parted`` command.
        partitions (list): The partitions found in the output, as Partition
            objects.
        boot_partition (Partition): the first partition marked as bootable,
            or ``None`` if one was not found.

    Raises:
        ParseException: Raised if ``parted`` output indicates "error" or
            "warning" in first line, or if "disk" field is not present, or if
            there is an error parsing the data.
        ValueError: Raised if there is an error parsing the partition table.
    """

    @property
    def disk(self):
        """str: Disk information."""
        return self.data['disk']

    @property
    def logical_sector_size(self):
        """str: Logical part of sector size."""
        if self._sector_size:
            return self._sector_size[0]

    @property
    def physical_sector_size(self):
        """str: Physical part of sector size."""
        if self._sector_size:
            return self._sector_size[1]

    def get(self, item):
        """Returns a value for the specified ``item`` key."""
        return self.data.get(item)

    def parse_content(self, content):
        # If device was not present output is error message
        if content[0].startswith("Error") or content[0].startswith("Warning"):
            raise ParseException("PartedL content indicates an error %s" % content[0])

        dev_info = {}
        table_lines = []
        for line in content:
            if not line.strip():
                continue
            if ':' in line:
                label_value = line.split(':')
                label = label_value[0].strip().lower()
                if len(label_value) == 2:
                    value = label_value[1].strip()
                    value = value if value else None
                    # Single word labels
                    if ' ' not in label:
                        dev_info[label] = value
                    else:
                        if label.startswith("disk") and '/' in label:
                            disk_parts = label.split()
                            dev_info['disk'] = disk_parts[1].strip()
                            dev_info['size'] = value
                        elif label.startswith("sector"):
                            dev_info['sector_size'] = value
                        else:
                            label = label.replace(' ', '_')
                            dev_info[label] = value
            else:
                table_lines.append(line)

        if 'disk' not in dev_info:
            raise ParseException("PartedL unable to locate Disk in content")

        # Now construct the partition table from the fixed table
        partitions = []
        if table_lines:
            table_lines[0] = table_lines[0].replace('File system', 'File_system').lower()
            partitions = parse_fixed_table(table_lines)

        self.partitions = [Partition(n) for n in partitions]
        self.boot_partition = None
        self._sector_size = None
        # If we got any partitions, find the first boot partition
        for part in partitions:
            if 'flags' in part and 'boot' in part['flags']:
                self.boot_partition = Partition(part)
                break
        self.data = dev_info
        if 'sector_size' in self.data:
            self._sector_size = self.data['sector_size'].split('/', 1)
            if len(self._sector_size) != 2:
                self._sector_size = None



import pytest
from mock.mock import patch
from insights.core.dr import SkipComponent
from insights.core.context import HostContext
from insights.specs.datasources.httpd import httpd_cmds
from insights.combiners.ps import Ps


# The ``get_running_commands()`` is tested in:
# - insights/tests/datasources/test_get_running_commands.py
# Here, we do not test it
@patch('insights.specs.datasources.httpd.get_running_commands')
def test_httpd_cmds(run_cmds):
    broker = {Ps: None, HostContext: None}
    httpds = ['/usr/sbin/httpd/', '/opt/rh/httpd24/root/usr/sbin/httpd']
    run_cmds.return_value = httpds
    result = httpd_cmds(broker)
    assert result == httpds

    run_cmds.return_value = []
    with pytest.raises(SkipComponent):
        httpd_cmds(broker)



"""
Samba status commands
=====================

This module provides processing for the ``smbstatus`` command using the
following parsers:

SmbstatusS - command ``/usr/bin/smbstatus -S``
----------------------------------------------

Smbstatusp - command ``/usr/bin/smbstatus -p``
----------------------------------------------
"""

from .. import parser, get_active_lines, CommandParser
from insights.parsers import ParseException, SkipException
from . import parse_fixed_table
from insights.specs import Specs


class Statuslist(CommandParser):
    """Base class implementing shared code."""

    def __len__(self):
        return len(self.data)

    def __iter__(self):
        for row in self.data:
            yield row

    def parse_content(self, content):
        new_content = get_active_lines(content, '-----------')
        if not content:
            raise SkipException("Empty content.")
        if len(content) == 1:
            raise ParseException("There is no useful parsed data in the content: '{0}'".format(content))
        return new_content


@parser(Specs.smbstatus_S)
class SmbstatusS(Statuslist):
    """
    Class ``SmbstatusS`` parses the output of the ``smbstatus -S`` command.

    Sample output of this command looks like::

        Service      pid     Machine       Connected at                     Encryption   Signing
        ----------------------------------------------------------------------------------------
        share_test   13668   10.66.208.149 Wed Sep 27 10:33:55 AM 2017 CST  -            -

    The format of ``smbstatus -S`` is like table, and function `parse_fixed_table`
    could parse it.

    Examples:
        >>> smbstatuss_info.data[0] == {'Signing': '-', 'Service': 'share_test', 'Encryption': '-', 'pid': '13668', 'Machine': '10.66.208.149', 'Connected_at': 'Wed Sep 27 10:33:55 AM 2017 CST'}
        True
        >>> smbstatuss_info.data[0]['pid']
        '13668'

    Raises:
        ParseException: When there is no usefull data or the input content is
            empty,  or does contain the header line.

    Attributes:
        data (list): List of dicts, where the keys in each dict are the column
            headers and each item in the list represents a connection.
    """

    def parse_content(self, content):
        content = super(SmbstatusS, self).parse_content(content)
        if not content[0].startswith('Service '):
            raise ParseException("Cannot find the header line.")
        self.data = parse_fixed_table(content, header_substitute=[('Connected at', 'Connected_at')])


@parser(Specs.smbstatus_p)
class Smbstatusp(Statuslist):
    """
    Class ``Smbstatusp`` parses the output of the ``smbstatus -p`` command.

    Sample output of this command looks like::

        Samba version 4.6.2
        PID     Username     Group        Machine                                   Protocol Version  Encryption           Signing
        --------------------------------------------------------------------------------------------------------------------------
        12668   testsmb       testsmb       10.66.208.149 (ipv4:10.66.208.149:44376)  SMB2_02           -                    -

    The format of ``smbstatus -p`` is like table, and function `parse_fixed_table`
    could parse it.

    Examples:
        >>> smbstatusp_info.data[0] == {'Username': 'testsmb', 'Signing': '-', 'Group': 'testsmb', 'Encryption': '-', 'PID': '12668', 'Machine': '10.66.208.149 (ipv4:10.66.208.149:44376)', 'Protocol_Version': 'SMB2_02'}
        True
        >>> smbstatusp_info.data[0]['PID']
        '12668'

    Raises:
        ParseException: When there is no usefull data or the input content is
            empty,  or does contain the header line.

    Attributes:
        data (list): List of dicts, where the keys in each dict are the column
            headers and each item in the list represents a connection.
    """

    def parse_content(self, content):
        content = super(Smbstatusp, self).parse_content(content)
        if not any(l.startswith('PID ') for l in content):
            raise ParseException("Cannot find the header line.")
        self.data = parse_fixed_table(content, heading_ignore=["PID     Username"],
                                      header_substitute=[('Protocol Version', 'Protocol_Version')])



"""
Handle adding files and preparing the archive for upload
"""
from __future__ import absolute_import
import glob
from signal import SIGTERM, signal
import sys
import time
import os
import shutil
import subprocess
import shlex
import logging
import tempfile
import re
import atexit

from .utilities import determine_hostname, _expand_paths, write_data_to_file
from .insights_spec import InsightsFile, InsightsCommand

logger = logging.getLogger(__name__)


class InsightsArchive(object):
    """
    This class is an interface for adding command output
    and files to the insights archive

    Attributes:
        config          - an InsightsConfig object
        tmp_dir         - a temporary directory in /var/tmp
        archive_dir     - location to collect archive data inside tmp_dir
        archive_tmp_dir - a temporary directory to write the final archive file
        archive_name    - filename of the archive and archive_dir
        cmd_dir         - insights_commands directory inside archive_dir
        compressor      - tar compression flag to use
        tar_file        - path of the final archive file
    """
    def __init__(self, config):
        """
        Initialize the Insights Archive
        """
        self.config = config
        self.cleanup_previous_archive()
        # input this to core collector as `tmp_path`
        self.tmp_dir = tempfile.mkdtemp(dir='/var/tmp/', prefix='insights-archive-')

        # we don't really need this anymore...
        self.archive_tmp_dir = tempfile.mkdtemp(dir='/var/tmp/', prefix='insights-archive-')

        # We should not hint the hostname in the archive if it has to be obfuscated
        if config.obfuscate_hostname:
            hostname = "localhost"
        else:
            hostname = determine_hostname()

        self.archive_name = ("insights-%s-%s" %
                             (hostname,
                              time.strftime("%Y%m%d%H%M%S")))

        # lazy create these, only if needed when certain
        #   functions are called
        # classic collection and compliance needs these
        # core collection will set "archive_dir" on its own
        self.archive_dir = None
        self.cmd_dir = None

        self.compressor = config.compressor
        self.archive_stored = None
        self.tar_file = None
        self.keep_archive_dir = '/var/cache/insights-client'
        atexit.register(self.cleanup_tmp)
        signal(SIGTERM, self.sigterm_handler)

    def create_archive_dir(self):
        """
        Create the archive directory if it is undefined or does not exist.
        """
        if self.archive_dir and os.path.exists(self.archive_dir):
            # attr defined and exists. move along
            return self.archive_dir

        archive_dir = os.path.join(self.tmp_dir, self.archive_name)
        if not os.path.exists(archive_dir):
            logger.debug('Creating archive directory %s...', archive_dir)
            os.makedirs(archive_dir, 0o700)
        self.archive_dir = archive_dir
        return self.archive_dir

    def create_command_dir(self):
        """
        Create the "insights_commands" dir
        """
        self.create_archive_dir()
        cmd_dir = os.path.join(self.archive_dir, "insights_commands")
        logger.debug('Creating command directory %s...', cmd_dir)
        if not os.path.exists(cmd_dir):
            os.makedirs(cmd_dir, 0o700)
        self.cmd_dir = cmd_dir
        return self.cmd_dir

    def get_full_archive_path(self, path):
        """
        Returns the full archive path
        """
        self.create_archive_dir()
        return os.path.join(self.archive_dir, path.lstrip('/'))

    def _copy_file(self, path):
        """
        Copy just a single file
        """
        full_path = self.get_full_archive_path(path)
        # Try to make the dir, eat exception if it fails
        try:
            os.makedirs(os.path.dirname(full_path))
        except OSError:
            pass
        logger.debug("Copying %s to %s", path, full_path)
        shutil.copyfile(path, full_path)
        return path

    def copy_file(self, path):
        """
        Copy a single file or regex, creating the necessary directories
        """
        if "*" in path:
            paths = _expand_paths(path)
            if paths:
                for path in paths:
                    self._copy_file(path)
        else:
            if os.path.isfile(path):
                return self._copy_file(path)
            else:
                logger.debug("File %s does not exist", path)
                return False

    def copy_dir(self, path):
        """
        Recursively copy directory
        """
        self.create_archive_dir()
        for directory in path:
            if os.path.isdir(path):
                full_path = os.path.join(self.archive_dir, directory.lstrip('/'))
                logger.debug("Copying %s to %s", directory, full_path)
                shutil.copytree(directory, full_path)
            else:
                logger.debug("Not a directory: %s", directory)
        return path

    def get_compression_flag(self, compressor):
        return {
            "gz": "z",
            "xz": "J",
            "bz2": "j",
            "none": ""
        }.get(compressor, "z")

    def create_tar_file(self):
        """
        Create tar file to be compressed
        """
        if not self.tmp_dir:
            # we should never get here but bail out if we do
            raise RuntimeError('Archive temporary directory not defined.')
        tar_file_name = os.path.join(self.archive_tmp_dir, self.archive_name)
        ext = "" if self.compressor == "none" else ".%s" % self.compressor
        tar_file_name = tar_file_name + ".tar" + ext
        logger.debug("Tar File: " + tar_file_name)
        return_code = subprocess.call(shlex.split("tar c%sfS %s -C %s ." % (
            self.get_compression_flag(self.compressor),
            tar_file_name, self.tmp_dir)),
            stderr=subprocess.PIPE)
        if (self.compressor in ["bz2", "xz"] and return_code != 0):
            logger.error("ERROR: %s compressor is not installed, cannot compress file", self.compressor)
            return None
        self.delete_archive_dir()
        logger.debug("Tar File Size: %s", str(os.path.getsize(tar_file_name)))
        self.tar_file = tar_file_name
        return tar_file_name

    def delete_tmp_dir(self):
        """
        Delete the entire tmp dir
        """
        if self.tmp_dir:
            logger.debug("Deleting: " + self.tmp_dir)
            shutil.rmtree(self.tmp_dir, True)

    def delete_archive_dir(self):
        """
        Delete the entire archive dir
        """
        if self.archive_dir:
            logger.debug("Deleting: " + self.archive_dir)
            shutil.rmtree(self.archive_dir, True)

    def delete_archive_file(self):
        """
        Delete the directory containing the constructed archive
        """
        if self.archive_tmp_dir:
            logger.debug("Deleting %s", self.archive_tmp_dir)
            shutil.rmtree(self.archive_tmp_dir, True)

    def add_to_archive(self, spec):
        '''
        Add files and commands to archive
        Use InsightsSpec.get_output() to get data
        '''
        ab_regex = [
            "^timeout: failed to run command .+: No such file or directory$",
            "^Missing Dependencies:"
        ]
        if isinstance(spec, InsightsCommand):
            archive_path = os.path.join(self.cmd_dir, spec.archive_path.lstrip('/'))
        if isinstance(spec, InsightsFile):
            archive_path = self.get_full_archive_path(spec.archive_path.lstrip('/'))
        output = spec.get_output()
        if output and not any(re.search(rg, output) for rg in ab_regex):
            write_data_to_file(output, archive_path)

    def add_metadata_to_archive(self, metadata, meta_path):
        '''
        Add metadata to archive
        '''
        archive_path = self.get_full_archive_path(meta_path.lstrip('/'))
        write_data_to_file(metadata, archive_path)

    def cleanup_tmp(self):
        '''
        Only used during built-in collection.
        Delete archive and tmp dirs on exit unless --keep-archive is specified
            and tar_file exists.
        '''
        if self.config.keep_archive and self.tar_file:
            self.storing_archive()
            if self.config.no_upload:
                logger.info('Archive saved at %s', self.archive_stored)
            else:
                logger.info('Insights archive retained in %s', self.archive_stored)
        self.delete_tmp_dir()

    def sigterm_handler(_signo, _stack_frame, _context):
        sys.exit(1)

    def cleanup_previous_archive(self):
        '''
        Used at the start, this will clean the temporary directory of previous killed runs
        '''
        for file in glob.glob('/var/tmp/insights-archive-*'):
            os.path.join('', file)
            logger.debug("Deleting previous archive %s", file)
            shutil.rmtree(file, True)

    def storing_archive(self):
        if not os.path.exists(self.keep_archive_dir):
            try:
                os.makedirs(self.keep_archive_dir)
            except OSError:
                logger.error('ERROR: Could not create %s', self.keep_archive_dir)
                raise

        archive_name = os.path.basename(self.tar_file)
        self.archive_stored = os.path.join(self.keep_archive_dir, archive_name)
        logger.info('Copying archive from %s to %s', self.tar_file, self.archive_stored)
        try:
            shutil.copyfile(self.tar_file, self.archive_stored)
        except OSError:
            # file exists already
            logger.error('ERROR: Could not stored archive to %s', self.archive_stored)
            raise



import pytest
from mock.mock import patch
from insights.core.dr import SkipComponent
from insights.specs.datasources.corosync import corosync_cmapctl_cmds
from insights.components.rhel_version import IsRhel6, IsRhel7, IsRhel8, IsRhel9

COROSYNC_CMD_RHEL7 = [
    "/usr/sbin/corosync-cmapctl",
    "/usr/sbin/corosync-cmapctl -d runtime.schedmiss.timestamp",
    "/usr/sbin/corosync-cmapctl -d runtime.schedmiss.delay"
]

COROSYNC_CMD_RHEL9 = [
    "/usr/sbin/corosync-cmapctl",
    "/usr/sbin/corosync-cmapctl -m stats",
    "/usr/sbin/corosync-cmapctl -C schedmiss"
]


@patch('insights.specs.datasources.corosync.os.path.exists')
def test_corosync_cmapctl_cmds(path_exists):
    path_exists.return_value = True
    broker = {IsRhel6: True}
    with pytest.raises(SkipComponent):
        corosync_cmapctl_cmds(broker)

    broker = {IsRhel7: True}
    result = corosync_cmapctl_cmds(broker)
    assert result == COROSYNC_CMD_RHEL7

    broker = {IsRhel8: True}
    result = corosync_cmapctl_cmds(broker)
    assert result == COROSYNC_CMD_RHEL9

    broker = {IsRhel9: True}
    result = corosync_cmapctl_cmds(broker)
    assert result == COROSYNC_CMD_RHEL9


@patch('insights.specs.datasources.corosync.os.path.exists')
def test_corosync_cmapctl_cmds_no_such_cmd(path_exists):
    path_exists.return_value = False
    broker = {IsRhel7: True}
    with pytest.raises(SkipComponent):
        corosync_cmapctl_cmds(broker)

    broker = {IsRhel9: True}
    with pytest.raises(SkipComponent):
        corosync_cmapctl_cmds(broker)



"""
NetworkManagerConfig - file ``/etc/NetworkManager/NetworkManager.conf``
=======================================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.networkmanager_conf)
class NetworkManagerConfig(IniConfigFile):
    """
    The ``/etc/NetworkManager/NetworkManager.conf`` file is in a standard '.ini' format,
    and this parser uses the IniConfigFile base class to read this.

    Given a file containing the following test data::

        [main]
        dhcp=dhclient

    Example:
        >>> type(networkmanager_config_obj)
        <class 'insights.parsers.networkmanager_config.NetworkManagerConfig'>
        >>> networkmanager_config_obj.get('main', 'dhcp') == 'dhclient'
        True
    """
    pass



"""
Ceilometer logs
===============

Module for parsing the log files for Ceilometer

CeilometerCentralLog - file ``/var/log/ceilometer/central.log``
---------------------------------------------------------------

CeilometerCollectorLog - file ``/var/log/ceilometer/collector.log``
-------------------------------------------------------------------

CeilometerComputeLog - file ``/var/log/ceilometer/compute.log``
---------------------------------------------------------------

.. note::
    Please refer to the super-class :class:`insights.core.LogFileOutput`

"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.ceilometer_central_log)
class CeilometerCentralLog(LogFileOutput):
    """Class for parsing ``/var/log/ceilometer/central.log`` file.

    Typical content of ``central.log`` file is::

        2016-11-09 14:38:08.484 31654 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.
        2016-11-09 14:38:09.711 31723 INFO ceilometer.declarative [-] Definitions: {'metric': [{'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.10.1.3.1', 'type': 'lambda x: float(str(x))', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.cpu.load.1min', 'unit': 'process'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.10.1.3.2', 'type': 'lambda x: float(str(x))', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.cpu.load.5min', 'unit': 'process'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.10.1.3.3', 'type': 'lambda x: float(str(x))', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.cpu.load.15min', 'unit': 'process'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.11.9.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.cpu.util', 'unit': '%'}, {'snmp_inspector': {'post_op': '_post_op_disk', 'oid': '1.3.6.1.4.1.2021.9.1.6', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'device': {'oid': '1.3.6.1.4.1.2021.9.1.3', 'type': 'str'}, 'path': {'oid': '1.3.6.1.4.1.2021.9.1.2', 'type': 'str'}}}, 'type': 'gauge', 'name': 'hardware.disk.size.total', 'unit': 'KB'}, {'snmp_inspector': {'post_op': '_post_op_disk', 'oid': '1.3.6.1.4.1.2021.9.1.8', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'device': {'oid': '1.3.6.1.4.1.2021.9.1.3', 'type': 'str'}, 'path': {'oid': '1.3.6.1.4.1.2021.9.1.2', 'type': 'str'}}}, 'type': 'gauge', 'name': 'hardware.disk.size.used', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.5.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.total', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.6.0', 'type': 'int', 'matching_type': 'type_exact', 'post_op': '_post_op_memory_avail_to_used'}, 'type': 'gauge', 'name': 'hardware.memory.used', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.3.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.swap.total', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.4.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.swap.avail', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.14.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.buffer', 'unit': 'KB'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.4.15.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.memory.cached', 'unit': 'KB'}, {'snmp_inspector': {'post_op': '_post_op_net', 'oid': '1.3.6.1.2.1.2.2.1.10', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'mac': {'oid': '1.3.6.1.2.1.2.2.1.6', 'type': "lambda x: x.prettyPrint().replace('0x', '')"}, 'speed': {'oid': '1.3.6.1.2.1.2.2.1.5', 'type': 'lambda x: int(x) / 8'}, 'name': {'oid': '1.3.6.1.2.1.2.2.1.2', 'type': 'str'}}}, 'type': 'cumulative', 'name': 'hardware.network.incoming.bytes', 'unit': 'B'}, {'snmp_inspector': {'post_op': '_post_op_net', 'oid': '1.3.6.1.2.1.2.2.1.16', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'mac': {'oid': '1.3.6.1.2.1.2.2.1.6', 'type': "lambda x: x.prettyPrint().replace('0x', '')"}, 'speed': {'oid': '1.3.6.1.2.1.2.2.1.5', 'type': 'lambda x: int(x) / 8'}, 'name': {'oid': '1.3.6.1.2.1.2.2.1.2', 'type': 'str'}}}, 'type': 'cumulative', 'name': 'hardware.network.outgoing.bytes', 'unit': 'B'}, {'snmp_inspector': {'post_op': '_post_op_net', 'oid': '1.3.6.1.2.1.2.2.1.20', 'type': 'int', 'matching_type': 'type_prefix', 'metadata': {'mac': {'oid': '1.3.6.1.2.1.2.2.1.6', 'type': "lambda x: x.prettyPrint().replace('0x', '')"}, 'speed': {'oid': '1.3.6.1.2.1.2.2.1.5', 'type': 'lambda x: int(x) / 8'}, 'name': {'oid': '1.3.6.1.2.1.2.2.1.2', 'type': 'str'}}}, 'type': 'cumulative', 'name': 'hardware.network.outgoing.errors', 'unit': 'packet'}, {'snmp_inspector': {'oid': '1.3.6.1.2.1.4.10.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'cumulative', 'name': 'hardware.network.ip.outgoing.datagrams', 'unit': 'datagrams'}, {'snmp_inspector': {'oid': '1.3.6.1.2.1.4.3.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'cumulative', 'name': 'hardware.network.ip.incoming.datagrams', 'unit': 'datagrams'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.11.11.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'gauge', 'name': 'hardware.system_stats.cpu.idle', 'unit': '%'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.11.57.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'cumulative', 'name': 'hardware.system_stats.io.outgoing.blocks', 'unit': 'blocks'}, {'snmp_inspector': {'oid': '1.3.6.1.4.1.2021.11.58.0', 'type': 'int', 'matching_type': 'type_exact'}, 'type': 'cumulative', 'name': 'hardware.system_stats.io.incoming.blocks', 'unit': 'blocks'}]}
        2016-11-09 14:38:09.986 31723 WARNING oslo_config.cfg [-] Option "rpc_backend" from group "DEFAULT" is deprecated for removal.  Its value may be silently ignored in the future.
        2016-11-09 14:38:10.040 31723 INFO ceilometer.pipeline [-] Config file: {'sources': [{'interval': 600, 'meters': ['*'], 'name': 'meter_source', 'sinks': ['meter_sink']}, {'interval': 600, 'meters': ['cpu'], 'name': 'cpu_source', 'sinks': ['cpu_sink', 'cpu_delta_sink']}, {'interval': 600, 'meters': ['disk.read.bytes', 'disk.read.requests', 'disk.write.bytes', 'disk.write.requests', 'disk.device.read.bytes', 'disk.device.read.requests', 'disk.device.write.bytes', 'disk.device.write.requests'], 'name': 'disk_source', 'sinks': ['disk_sink']}, {'interval': 600, 'meters': ['network.incoming.bytes', 'network.incoming.packets', 'network.outgoing.bytes', 'network.outgoing.packets'], 'name': 'network_source', 'sinks': ['network_sink']}], 'sinks': [{'publishers': ['notifier://'], 'transformers': None, 'name': 'meter_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'target': {'scale': '100.0 / (10**9 * (resource_metadata.cpu_number or 1))', 'type': 'gauge', 'name': 'cpu_util', 'unit': '%'}}}], 'name': 'cpu_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'delta', 'parameters': {'target': {'name': 'cpu.delta'}, 'growth_only': True}}], 'name': 'cpu_delta_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'source': {'map_from': {'name': '(disk\\.device|disk)\\.(read|write)\\.(bytes|requests)', 'unit': '(B|request)'}}, 'target': {'map_to': {'name': '\\1.\\2.\\3.rate', 'unit': '\\1/s'}, 'type': 'gauge'}}}], 'name': 'disk_sink'}, {'publishers': ['notifier://'], 'transformers': [{'name': 'rate_of_change', 'parameters': {'source': {'map_from': {'name': 'network\\.(incoming|outgoing)\\.(bytes|packets)', 'unit': '(B|packet)'}}, 'target': {'map_to': {'name': 'network.\\1.\\2.rate', 'unit': '\\1/s'}, 'type': 'gauge'}}}], 'name': 'network_sink'}]}
        2016-11-09 14:38:10.041 31723 INFO ceilometer.pipeline [-] detected decoupled pipeline config format
        2016-11-09 14:38:10.053 31723 INFO ceilometer.coordination [-] Coordination backend started successfully.
        2016-11-09 14:38:10.064 31723 INFO ceilometer.coordination [-] Joined partitioning group central-global
        2016-11-09 14:58:10.621 31723 INFO ceilometer.agent.manager [-] Skip pollster switch, no resources found this cycle
        2016-11-09 14:58:15.655 31723 INFO ceilometer.agent.manager [-] Skip pollster hardware.memory.used, no resources found this cycle
        2016-11-09 14:58:15.656 31723 INFO ceilometer.agent.manager [-] Skip pollster switch.port, no resources found this cycle
        2016-11-09 14:58:15.657 31723 INFO ceilometer.agent.manager [-] Skip pollster switch.port.receive.bytes, no resources found this cycle
        2016-11-09 14:58:15.657 31723 INFO ceilometer.agent.manager [-] Skip pollster hardware.system_stats.io.incoming.blocks, no resources found this cycle
        2016-11-09 14:58:17.027 31723 WARNING ceilometer.neutron_client [-] The resource could not be found.
    """
    pass


@parser(Specs.ceilometer_collector_log)
class CeilometerCollectorLog(LogFileOutput):
    """Class for parsing ``/var/log/ceilometer/collector.log`` file.

    Typical content of ``collector.log`` file is::

        2016-11-09 14:32:40.269 4204 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.
        2016-11-09 14:32:40.467 4259 INFO ceilometer.declarative [-] Definitions: {'resources': [{'metrics': ['identity.authenticate.success', 'identity.authenticate.pending', 'identity.authenticate.failure', 'identity.user.created', 'identity.user.deleted', 'identity.user.updated', 'identity.group.created', 'identity.group.deleted', 'identity.group.updated', 'identity.role.created', 'identity.role.deleted', 'identity.role.updated', 'identity.project.created', 'identity.project.deleted', 'identity.project.updated', 'identity.trust.created', 'identity.trust.deleted', 'identity.role_assignment.created', 'identity.role_assignment.deleted'], 'archive_policy': 'low', 'resource_type': 'identity'}, {'metrics': ['radosgw.objects', 'radosgw.objects.size', 'radosgw.objects.containers', 'radosgw.api.request', 'radosgw.containers.objects', 'radosgw.containers.objects.size'], 'resource_type': 'ceph_account'}, {'metrics': ['instance', 'memory', 'memory.usage', 'memory.resident', 'vcpus', 'cpu', 'cpu.delta', 'cpu_util', 'disk.root.size', 'disk.ephemeral.size', 'disk.read.requests', 'disk.read.requests.rate', 'disk.write.requests', 'disk.write.requests.rate', 'disk.read.bytes', 'disk.read.bytes.rate', 'disk.write.bytes', 'disk.write.bytes.rate', 'disk.latency', 'disk.iops', 'disk.capacity', 'disk.allocation', 'disk.usage'], 'event_associated_resources': {'instance_network_interface': '{"=": {"instance_id": "%s"}}', 'instance_disk': '{"=": {"instance_id": "%s"}}'}, 'event_delete': 'compute.instance.delete.start', 'attributes': {'display_name': 'resource_metadata.display_name', 'host': 'resource_metadata.host', 'image_ref': 'resource_metadata.image_ref', 'flavor_id': 'resource_metadata.(instance_flavor_id|(flavor.id))', 'server_group': 'resource_metadata.user_metadata.server_group'}, 'event_attributes': {'id': 'payload.instance_id'}, 'resource_type': 'instance'}, {'metrics': ['network.outgoing.packets.rate', 'network.incoming.packets.rate', 'network.outgoing.packets', 'network.incoming.packets', 'network.outgoing.bytes.rate', 'network.incoming.bytes.rate', 'network.outgoing.bytes', 'network.incoming.bytes'], 'attributes': {'instance_id': 'resource_metadata.instance_id', 'name': 'resource_metadata.vnic_name'}, 'resource_type': 'instance_network_interface'}, {'metrics': ['disk.device.read.requests', 'disk.device.read.requests.rate', 'disk.device.write.requests', 'disk.device.write.requests.rate', 'disk.device.read.bytes', 'disk.device.read.bytes.rate', 'disk.device.write.bytes', 'disk.device.write.bytes.rate', 'disk.device.latency', 'disk.device.iops', 'disk.device.capacity', 'disk.device.allocation', 'disk.device.usage'], 'attributes': {'instance_id': 'resource_metadata.instance_id', 'name': 'resource_metadata.disk_name'}, 'resource_type': 'instance_disk'}, {'metrics': ['image', 'image.size', 'image.download', 'image.serve'], 'attributes': {'container_format': 'resource_metadata.container_format', 'disk_format': 'resource_metadata.disk_format', 'name': 'resource_metadata.name'}, 'event_delete': 'image.delete', 'event_attributes': {'id': 'payload.resource_id'}, 'resource_type': 'image'}, {'metrics': ['hardware.ipmi.node.power', 'hardware.ipmi.node.temperature', 'hardware.ipmi.node.inlet_temperature', 'hardware.ipmi.node.outlet_temperature', 'hardware.ipmi.node.fan', 'hardware.ipmi.node.current', 'hardware.ipmi.node.voltage', 'hardware.ipmi.node.airflow', 'hardware.ipmi.node.cups', 'hardware.ipmi.node.cpu_util', 'hardware.ipmi.node.mem_util', 'hardware.ipmi.node.io_util'], 'resource_type': 'ipmi'}, {'metrics': ['bandwidth', 'network', 'network.create', 'network.update', 'subnet', 'subnet.create', 'subnet.update', 'port', 'port.create', 'port.update', 'router', 'router.create', 'router.update', 'ip.floating', 'ip.floating.create', 'ip.floating.update'], 'resource_type': 'network'}, {'metrics': ['stack.create', 'stack.update', 'stack.delete', 'stack.resume', 'stack.suspend'], 'resource_type': 'stack'}, {'metrics': ['storage.objects.incoming.bytes', 'storage.objects.outgoing.bytes', 'storage.api.request', 'storage.objects.size', 'storage.objects', 'storage.objects.containers', 'storage.containers.objects', 'storage.containers.objects.size'], 'resource_type': 'swift_account'}, {'metrics': ['volume', 'volume.size', 'volume.create', 'volume.delete', 'volume.update', 'volume.resize', 'volume.attach', 'volume.detach'], 'attributes': {'display_name': 'resource_metadata.display_name'}, 'resource_type': 'volume'}, {'metrics': ['hardware.cpu.load.1min', 'hardware.cpu.load.5min', 'hardware.cpu.load.15min', 'hardware.cpu.util', 'hardware.memory.total', 'hardware.memory.used', 'hardware.memory.swap.total', 'hardware.memory.swap.avail', 'hardware.memory.buffer', 'hardware.memory.cached', 'hardware.network.ip.outgoing.datagrams', 'hardware.network.ip.incoming.datagrams', 'hardware.system_stats.cpu.idle', 'hardware.system_stats.io.outgoing.blocks', 'hardware.system_stats.io.incoming.blocks'], 'attributes': {'host_name': 'resource_metadata.resource_url'}, 'resource_type': 'host'}, {'metrics': ['hardware.disk.size.total', 'hardware.disk.size.used'], 'attributes': {'host_name': 'resource_metadata.resource_url', 'device_name': 'resource_metadata.device'}, 'resource_type': 'host_disk'}, {'metrics': ['hardware.network.incoming.bytes', 'hardware.network.outgoing.bytes', 'hardware.network.outgoing.errors'], 'attributes': {'host_name': 'resource_metadata.resource_url', 'device_name': 'resource_metadata.name'}, 'resource_type': 'host_network_interface'}]}
        2016-11-09 14:32:41.099 4259 WARNING oslo_config.cfg [-] Option "max_retries" from group "database" is deprecated. Use option "max_retries" from group "storage".
        2016-11-09 14:36:35.464 4204 INFO cotyledon [-] Caught SIGTERM signal, graceful exiting of master process
        2016-11-09 14:36:35.465 4259 INFO cotyledon [-] Caught signal (15) during service initialisation, delaying it
        2016-11-09 14:38:07.280 31638 WARNING oslo_reports.guru_meditation_report [-] Guru meditation now registers SIGUSR1 and SIGUSR2 by default for backward compatibility. SIGUSR1 will no longer be registered in a future release, so please use SIGUSR2 to generate reports.
    """
    pass


@parser(Specs.ceilometer_compute_log)
class CeilometerComputeLog(LogFileOutput):
    """Class for parsing ``/var/log/ceilometer/compute.log`` file.

    Typical content of ``compute.log`` file is::

        2018-01-12 21:00:02.939 49455 INFO ceilometer.agent.manager [-] Polling pollster network.outgoing.packets in the context of some_pollsters
        2018-01-12 21:00:02.950 49455 INFO ceilometer.agent.manager [-] Polling pollster memory.usage in the context of some_pollsters
        2018-01-12 21:00:02.953 49455 WARNING ceilometer.compute.pollsters.memory [-] Cannot inspect data of MemoryUsagePollster for xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx, non-fatal reason: Failed to inspect memory usage of instance <name=instance-name1, id=xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx>, can not get info from libvirt.
        2018-01-12 21:00:02.957 49455 WARNING ceilometer.compute.pollsters.memory [-] Cannot inspect data of MemoryUsagePollster for yyyyyyyy-yyyy-yyyy-yyyy-yyyyyyyyyyyy, non-fatal reason: Failed to inspect memory usage of instance <name=instance-name2, id=yyyyyyyy-yyyy-yyyy-yyyy-yyyyyyyyyyyy>, can not get info from libvirt.
        2018-01-12 21:00:02.963 49455 WARNING ceilometer.compute.pollsters.memory [-] Cannot inspect data of MemoryUsagePollster for zzzzzzzz-zzzz-zzzz-zzzz-zzzzzzzzzzzz, non-fatal reason: Failed to inspect memory usage of instance <name=instance-name3, id=zzzzzzzz-zzzz-zzzz-zzzz-zzzzzzzzzzzz>, can not get info from libvirt.
        2018-01-12 21:00:02.970 49455 INFO ceilometer.agent.manager [-] Polling pollster disk.write.requests in the context of some_pollsters
        2018-01-12 21:00:02.976 49455 INFO ceilometer.agent.manager [-] Polling pollster network.incoming.packets in the context of some_pollsters
        2018-01-12 21:00:02.981 49455 INFO ceilometer.agent.manager [-] Polling pollster cpu in the context of some_pollsters
        2018-01-12 21:00:03.014 49455 INFO ceilometer.agent.manager [-] Polling pollster network.incoming.bytes in the context of some_pollsters
        2018-01-12 21:00:03.020 49455 INFO ceilometer.agent.manager [-] Polling pollster disk.read.requests in the context of some_pollsters
        2018-01-12 21:00:03.041 49455 INFO ceilometer.agent.manager [-] Polling pollster network.outgoing.bytes in the context of some_pollsters
        2018-01-12 21:00:03.062 49455 INFO ceilometer.agent.manager [-] Polling pollster disk.write.bytes in the context of some_pollsters
    """
    pass



"""
LsvmBus - Command ``lsvmbus -vv``
=================================

This module parses the output of the command ``lsvmbus -vv``.
"""
import re

from insights.util import deprecated
from insights import parser, CommandParser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.lsvmbus)
class LsvmBus(CommandParser):
    """Parse the output of ``lsvmbus -vv`` as list.

    .. warning::
        This parser class is deprecated, please use
        :py:class:`insights.combiner.sys_vmbus_devices.SysVmBusDeviceInfo` instead.

    Typical output::

        VMBUS ID 18: Class_ID = {44c4f61d-4444-4400-9d52-802e27ede19f} - PCI Express pass-through
                Device_ID = {47505500-0001-0000-3130-444531303244}
                Sysfs path: /sys/bus/vmbus/devices/47505500-0001-0000-3130-444531303244
                Rel_ID=18, target_cpu=0
        VMBUS ID 26: Class_ID = {44c4f61d-4444-4400-9d52-802e27ede19f} - PCI Express pass-through
                Device_ID = {47505500-0002-0000-3130-444531303244}
                Sysfs path: /sys/bus/vmbus/devices/47505500-0002-0000-3130-444531303244
                Rel_ID=26, target_cpu=0
        VMBUS ID 73: Class_ID = {44c4f61d-4444-4400-9d52-802e27ede19f} - PCI Express pass-through
                Device_ID = {47505500-0003-0001-3130-444531303244}
                Sysfs path: /sys/bus/vmbus/devices/47505500-0003-0001-3130-444531303244
                Rel_ID=73, target_cpu=0
        VMBUS ID 74: Class_ID = {44c4f61d-4444-4400-9d52-802e27ede19f} - PCI Express pass-through
                Device_ID = {47505500-0004-0001-3130-444531303244}
                Sysfs path: /sys/bus/vmbus/devices/47505500-0004-0001-3130-444531303244
                Rel_ID=74, target_cpu=0

    Examples:

        >>> assert len(lsvmbus.devices) == 4
        >>> assert lsvmbus.devices[0].get('vmbus_id', None) == '18'
        >>> assert lsvmbus.devices[0].get('device_id', None) == '47505500-0001-0000-3130-444531303244'
        >>> assert lsvmbus.devices[0].get('rel_id', None) == '18'
        >>> assert lsvmbus.devices[0].get('sysfs_path', None) == '/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531303244'
        >>> assert lsvmbus.devices[0].get('target_cpu', None) == '0'

    Attributes:
        devices (list): List of ``dict`` for each device. For example::

                        [
                         {
                          'vmbus_id': '18',
                          'class_id': '44c4f61d-4444-4400-9d52-802e27ede19f',
                          'type': 'PCI Express pass-through',
                          'device_id': '47505500-0001-0000-3130-444531303244',
                          'sysfs_path': '/sys/bus/vmbus/devices/47505500-0001-0000-3130-444531303244',
                          'rel_id': '18',
                          'target_cpu': '0'
                         },
                         {...}
                        ]

    """
    def __init__(self, *args, **kwargs):
        deprecated(LsvmBus, "Use the SysVmBusDeviceInfo combiner instead.", "3.0.300")
        super(LsvmBus, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        if not content:
            raise SkipException('No content.')
        self.devices = []
        parts = zip(*(iter(content),) * 4)
        patrn_vmbusid = re.compile(r"VMBUS ID (\d+)")
        patrn_type = re.compile("- (.*)")
        patrn_classid = re.compile("Class_ID = {(.*)}")
        patrn_deviceid = re.compile("Device_ID = {(.*)}")
        patrn_sysfspath = re.compile("Sysfs path: (.*)")
        patrn_relid = re.compile(r"Rel_ID=(\d+)")
        patrn_targetcpu = re.compile(r"target_cpu=(\d+)")
        for part in parts:
            self.devices.append(
                {
                    'vmbus_id': patrn_vmbusid.search(part[0]).groups()[0],
                    'class_id': patrn_classid.search(part[0]).groups()[0],
                    'type': patrn_type.search(part[0]).groups()[0],
                    'device_id': patrn_deviceid.search(part[1]).groups()[0],
                    'sysfs_path': patrn_sysfspath.search(part[2]).groups()[0],
                    'rel_id': patrn_relid.search(part[3]).groups()[0],
                    'target_cpu': patrn_targetcpu.search(part[3]).groups()[0],
                }
            )



import mock
import pytest

from mock.mock import Mock

from insights.core.dr import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.system_user_dirs import LocalSpecs, system_user_dirs

RPM_CMD = """
sssd-ldap; drwxr-xr-x; apache; root
ca-certificates; drwxrwxr-x; root; apache
kmod; drwxr-xrwx; root; root
libbsd; dr-xr-xr-x; apache; root
libbsd; drwxr-xr-x; root; apache
libbsd; drwxrwxr-x; root; root
libbsd; -rwxr-xr-x; apache; root
libbsd; lrwxrwxrwx; apache; apache
""".strip()

RPM_EXPECTED = ["ca-certificates", "kmod", "sssd-ldap"]

RPM_BAD_CMD = "bash: rpm: command not found..."

RPM_EMPTY_CMD = ""

RELATIVE_PATH = "insights_commands/system_user_dirs"


def get_users():
    return ["apache"]


def get_groups(users):
    return ["apache"]


@mock.patch("insights.specs.datasources.system_user_dirs.get_users", get_users)
@mock.patch("insights.specs.datasources.system_user_dirs.get_groups", get_groups)
def test_rpm():
    rpm_args = Mock()
    rpm_args.content = RPM_CMD.splitlines()
    broker = {LocalSpecs.rpm_args: rpm_args}

    result = system_user_dirs(broker)
    expected = DatasourceProvider(content=RPM_EXPECTED, relative_path=RELATIVE_PATH)
    assert result
    assert isinstance(result, DatasourceProvider)
    assert sorted(result.content) == sorted(expected.content)
    assert result.relative_path == expected.relative_path


@pytest.mark.parametrize("no_rpm", [RPM_BAD_CMD, RPM_EMPTY_CMD])
def test_no_rpm(no_rpm):
    rpm_args = Mock()
    rpm_args.content = no_rpm.splitlines()
    broker = {LocalSpecs.rpm_args: rpm_args}

    with pytest.raises(SkipComponent):
        system_user_dirs(broker)



"""
CrictlLogs - commands `crictl logs -t <container's_ID>``
========================================================
"""

from insights.core import LogFileOutput
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.crictl_logs)
class CrictlLogs(LogFileOutput):
    """
    Class for parsing the output of commands ``crictl logs -t <container's_ID>``.

    Sample input data looks like::

        2021-12-21T11:12:45.854971114+01:00 Successfully copied files in /usr/src/multus-cni/rhel7/bin/ to /host/opt/cni/bin/
        2021-12-21T11:12:45.995998017+01:00 2021-12-21T10:12:45+00:00 WARN: {unknown parameter "-"}
        2021-12-21T11:12:46.008998978+01:00 2021-12-21T10:12:46+00:00 Entrypoint skipped copying Multus binary.
        2021-12-21T11:12:46.081427544+01:00 2021-12-21T10:12:46+00:00 Attempting to find master plugin configuration, attempt 0

    Note:
        Please refer to its super-class :class:`insights.core.LogFileOutput`.

    Examples:
        >>> len(logs.get('skipped copying Multus binary'))
        1
    """
    pass



"""
PCSConfig - command ``pcs config``
==================================

This module provides class ``PCSConfig`` for parsing output of ``pcs config`` command.

Typical ``/usr/sbin/pcs config`` output looks something like::

    Cluster Name: cluster-1
    Corosync Nodes:
     node-1 node-2
    Pacemaker Nodes:
     node-1 node-2

    Resources:
     Clone: clone-1
     Meta Attrs: interleave=true ordered=true
     Resource: res-1 (class=ocf provider=pacemaker type=controld)
      Operations: start interval=0s timeout=90 (dlm-start-interval-0s)
                  stop interval=0s timeout=100 (dlm-stop-interval-0s)
                  monitor interval=30s on-fail=fence (dlm-monitor-interval-30s)
     Group: grp-1
     Resource: res-1 (class=ocf provider=heartbeat type=IPaddr2)
      Attributes: ip=10.0.0.1 cidr_netmask=32
      Operations: monitor interval=120s (ip_monitor-interval-120s)
                  start interval=0s timeout=20s (ip_-start-interval-0s)
                  stop interval=0s timeout=20s (ip_-stop-interval-0s)

    Stonith Devices:
    Fencing Levels:

    Location Constraints:
    Resource: fence-1
        Disabled on: res-mgt (score:-INFINITY) (id:location-fence-1--INFINITY)
    Resource: res-1
        Enabled on: res-mcast (score:INFINITY) (role: Started) (id:cli-prefer-res)
    Ordering Constraints:
    Colocation Constraints:

    Resources Defaults:
     resource-stickiness: 100
     migration-threshold: 3
    Operations Defaults:
     No defaults set

    Cluster Properties:
     cluster-infrastructure: corosync
     cluster-name: cluster-1
     dc-version: 1.1.13-10.el7_2.4-44eb2dd
     have-watchdog: false
     no-quorum-policy: ignore
     stonith-enable: true
     stonith-enabled: false

The class provides attribute ``data`` as dictionary with lines parsed line by line based on keys, which are
the key words of the output.
Information in keys ``Corosync Nodes`` and ``Pacemaker Nodes`` is parsed in one line.
The get method ``get(str)`` provides lines from ``data`` based on given key.

Examples:
    >>> pcs_config.get("Cluster Name")
    'cluster-1'
    >>> pcs_config.get("Corosync Nodes")
    ['node-1', 'node-2']
    >>> pcs_config.get("Cluster Properties")
    ['cluster-infrastructure: corosync', 'cluster-name: cluster-1', 'dc-version: 1.1.13-10.el7_2.4-44eb2dd', 'have-watchdog: false', 'no-quorum-policy: ignore', 'stonith-enable: true', 'stonith-enabled: false']
    >>> pcs_config.get("Colocation Constraints")
    ['clone-1 with clone-x (score:INFINITY) (id:clone-INFINITY)', 'clone-2 with clone-x (score:INFINITY) (id:clone-INFINITY)']
    >>> 'have-watchdog' in pcs_config.cluster_properties
    True
    >>> pcs_config.cluster_properties.get('have-watchdog')
    'false'
"""


from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.pcs_config)
class PCSConfig(CommandParser):
    """Class to process the output of ``pcs config`` command.

    Attributes:
        cluster_properties (dict): A dictionary containing all cluster properties key, value
        data (dict): A dictionary containing all line. Keys sorted based on keywords of the output.
            the form::

                {
                    "Cluster Name": "cluster-1",
                    "Corosync Nodes": [
                        "node-1",
                        "node-2"
                    ],
                    "Pacemaker Nodes": [
                        "node-1",
                        "node-2"
                    ],
                    "Resources": [
                        "Clone: clone-1",
                        "Meta Attrs: interleave=true ordered=true",
                        "Resource: res-1 (class=ocf provider=pacemaker type=controld)",
                        "Operations: start interval=0s timeout=90 (dlm-start-interval-0s)",
                        "stop interval=0s timeout=100 (dlm-stop-interval-0s)",
                        "monitor interval=30s on-fail=fence (dlm-monitor-interval-30s)"
                    ],
                    "Cluster Properties": [
                        "cluster-infrastructure: corosync",
                        "cluster-name: cluster-1"
                    ]
                }
    """

    def parse_content(self, content):
        self.data = {}
        self.cluster_properties = {}
        dc_key = ""
        list_data = []
        in_line_tuple = ("Corosync Nodes", "Pacemaker Nodes")
        for i, line in enumerate(content):
            if line.startswith("Cluster Name:"):
                self.data["Cluster Name"] = line.split("Cluster Name:")[-1].lstrip()
                continue
            if line == "":
                if not dc_key == "":
                    self.data[dc_key] = list_data
                    list_data = []
                    dc_key = ""
                continue
            if (len(line) - line.find(":") == 1) and (len(line) - len(line.lstrip()) == 0):
                if not dc_key == "":
                    self.data[dc_key] = list_data
                    list_data = []
                dc_key = line.split(":")[0]
                continue
            list_data.append(line.lstrip())
            # build cluster properties dict
            if dc_key == 'Cluster Properties':
                cp_key, _, cp_value = line.partition(":")
                self.cluster_properties[cp_key.strip()] = cp_value.strip()
        if not dc_key == "" and dc_key not in self.data.keys():
            self.data[dc_key] = list_data
        for in_line in in_line_tuple:
            if isinstance(self.data.get(in_line), list):
                self.data[in_line] = self.data[in_line][0].split()

    def get(self, key):
        return self.data.get(key)



"""
VersionInfo - file ``version_info``
===================================

The version of the insights core and insights client that the archive used.

"""
from insights import JSONParser, parser
from insights.specs import Specs


@parser(Specs.version_info)
class VersionInfo(JSONParser):
    """
    This parser parses the ``version_info`` file generated by the
    ``insights-client`` command.

    Typical content of this file is::

        {"core_version": "3.0.203-1", "client_version": "3.1.1"}

    .. note::

        The :attr:`client_version` provided by this Parser is a short version
        only, to get the full version of the ``insights-client`` package,
        please use the :class:`insights.parsers.installed_rpms.InstalledRpms`
        Parser instead.

    Examples:
        >>> ver.core_version == '3.0.203-1'
        True
        >>> ver.client_version == '3.1.1'
        True
    """
    @property
    def core_version(self):
        """
        Returns:
            (str): The version of the insights core.
        """
        return self.data['core_version']

    @property
    def client_version(self):
        """
        Returns:
            (str): The version of the insights client.

        .. note::

            This attribute returns a short version of the insights client only,
            to get the full version of the ``insights-client`` package, please
            use the :class:`insights.parsers.installed_rpms.InstalledRpms` Parser
            instead.
        """
        return self.data['client_version']



from insights.tests import context_wrap
from insights.parsers.auditctl_status import AuditctlStatus
from insights.parsers import ParseException
import pytest

NORMAL_AUDS_RHEL6 = """
AUDIT_STATUS: enabled=1 flag=1 pid=1483 rate_limit=0 backlog_limit=8192 lost=3 backlog=0
""".strip()

NORMAL_AUDS_RHEL7 = """
enabled 1
failure 1
pid 947
rate_limit 0
backlog_limit 320
lost 0
backlog 0
loginuid_immutable 1 locked
""".strip()

BLANK_INPUT_SAMPLE = """
""".strip()

BAD_INPUT_SAMPLE = """
Unknown: type=0, len=0
""".strip()

BAD_INPUT_MIX = """
Unknown: type=0, len=0
enabled 1
""".strip()


def test_normal_auds_rhel6():
    auds = AuditctlStatus(context_wrap(NORMAL_AUDS_RHEL6))
    assert "enabled" in auds
    assert "loginuid_immutable" not in auds
    assert auds['pid'] == 1483


def test_normal_auds_rhel7():
    auds = AuditctlStatus(context_wrap(NORMAL_AUDS_RHEL7))
    assert "loginuid_immutable" in auds
    assert auds['loginuid_immutable'] == "1 locked"
    assert auds['failure'] == 1
    assert auds.get('nonexists') is None


def test_auds_blank_input():
    ctx = context_wrap(BLANK_INPUT_SAMPLE)
    with pytest.raises(ParseException) as sc:
        AuditctlStatus(ctx)
    assert "Input content is empty." in str(sc)


def test_auds_bad_input():
    auds = AuditctlStatus(context_wrap(BAD_INPUT_SAMPLE))
    assert auds.data == {}


def test_auds_bad_input_mix():
    auds = AuditctlStatus(context_wrap(BAD_INPUT_MIX))
    assert "enabled" in auds



"""
nmcli_dev_show command
======================
As there are three different file paths in different sos packages,
create this combiner to fix this issue.
"""

from insights.core.plugins import combiner
from insights.parsers.nmcli import NmcliDevShow, NmcliDevShowSos, SkipException


@combiner([NmcliDevShow, NmcliDevShowSos])
class AllNmcliDevShow(dict):
    """
    Combiner to combine return values from parser NmcliDevShow into one dict

    Examples:
        >>> allnmclidevshow['eth0']['TYPE']
        'ethernet'
        >>> allnmclidevshow.connected_devices
        ['eth0']
    """

    def __init__(self, nmclidevshow, nmclidevshowsos):
        data = {}
        self._con_dev = []

        if nmclidevshow:
            data.update(nmclidevshow)
            self._con_dev = nmclidevshow.connected_devices
        elif nmclidevshowsos:
            for item in nmclidevshowsos:
                data.update(item)
                self._con_dev.extend(item.connected_devices)

        if not data:
            raise SkipException()

        super(AllNmcliDevShow, self).__init__()
        self.update(data)

    @property
    def connected_devices(self):
        """(list): The list of devices who's state is connected and managed by NetworkManager"""
        return self._con_dev

    @property
    def data(self):
        """(dict): Dict with the device name as the key and NmcliDevShow details as the value."""
        return self



import doctest

from insights.parsers import foreman_tasks_config
from insights.parsers.foreman_tasks_config import ForemanTasksConfig
from insights.tests import context_wrap

FOREMAN_TASKS_CONFIG = """
FOREMAN_USER=foreman
BUNDLER_EXT_HOME=/usr/share/foreman
RAILS_ENV=production
FOREMAN_LOGGING=warn
FOREMAN_LOGGING_SQL=warn
FOREMAN_TASK_PARAMS="-p foreman"
FOREMAN_LOG_DIR=/var/log/foreman

RUBY_GC_MALLOC_LIMIT=4000100
RUBY_GC_MALLOC_LIMIT_MAX=16000100
RUBY_GC_MALLOC_LIMIT_GROWTH_FACTOR=1.1
RUBY_GC_OLDMALLOC_LIMIT=16000100
RUBY_GC_OLDMALLOC_LIMIT_MAX=16000100

#Set the number of executors you want to run
#EXECUTORS_COUNT=1

#Set memory limit for executor process, before it's restarted automatically
#EXECUTOR_MEMORY_LIMIT=2gb

#Set delay before first memory polling to let executor initialize (in sec)
#EXECUTOR_MEMORY_MONITOR_DELAY=7200 #default: 2 hours

#Set memory polling interval, process memory will be checked every N seconds.
#EXECUTOR_MEMORY_MONITOR_INTERVAL=60
"""


def test_foreman_tasks_config():
    foreman_tasks_config = ForemanTasksConfig(context_wrap(FOREMAN_TASKS_CONFIG)).data
    assert foreman_tasks_config["RAILS_ENV"] == 'production'
    assert foreman_tasks_config.get("FOREMAN_LOGGING") == 'warn'
    assert len(foreman_tasks_config) == 12


def test_foreman_tasks_config_doc_examples():
    env = {
        'foreman_tasks_config': ForemanTasksConfig(context_wrap(FOREMAN_TASKS_CONFIG)).data,
    }
    failed, total = doctest.testmod(foreman_tasks_config, globs=env)
    assert failed == 0



"""
NamedConf parser - file ``/etc/named.conf``
===========================================

NamedConf parser the file named configuration file.
Named is a name server used by BIND.
"""

from insights.specs import Specs
from insights.core.plugins import parser
from insights.parsers import SkipException
from insights.parsers.named_checkconf import NamedCheckconf


@parser(Specs.named_conf)
class NamedConf(NamedCheckconf):
    """
    Class for parsing the file ``/etc/named.conf```, We use class ``NamedCheckConf`` to parse most
    of the named.conf configurations and class ``NamedConf`` to parse the `include` directives and
    the `allow-recursion` values.

    .. note::
        Please refer to the super-class :py:class:`insights.parsers.named_checkconf:NamedCheckConf`
        for more usage information.

    Attributes:
        includes (list): List of files in 'include' section.
        allow_recursion_address (list): List of address in 'allow-recursion' section.

    Raises:
        SkipException: When content is empty or cannot be parsed.

    Examples:
        >>> named_conf.includes
        ['/etc/crypto-policies/back-ends/bind.config']
        >>> named_conf.allow_recursion_address
        ['localnets', 'localhost', '192.168.10.1/24']
    """

    def parse_content(self, content):
        includes = []
        allow_recursion_address = []
        super(NamedConf, self).parse_content(content)

        try:
            for line in content:
                line = line.strip()
                if line.startswith('include ') and ';' in line:
                    includes.append(line.split(';')[0].replace('"', '').split()[1])
                if line.startswith('allow-recursion ') and line.endswith(';'):
                    allow_recursion_address.extend(i.strip(';') for i in line.split(None, 1)[1].strip('{}; ').split())

        except IndexError:
            raise SkipException("Syntax error of include directive")

        self.includes = includes
        self.allow_recursion_address = allow_recursion_address



"""
insights.parsr.query defines a common data model and query language for parsers
created with ``insights.parsr`` to target.

The model allows duplicate keys, and it allows values with *unnamed* attributes
and recursive substructure. This is a common model for many kinds of
configuration.

Simple key/value pairs can be represented as a key with a value that has a
single attribute. Most dictionary shapes used to represent configuration are
made of keys with simple values (key/single attr), lists of simple values
(key/multiple attrs), or nested dictionaries (key/substructure).

Something like XML allows duplicate keys, and it allows values to have named
attributes and substructure. This module doesn't cover that case.

:py:class:`Entry`, :py:class:`Directive`, :py:class:`Section`, and
:py:class:`Result` have overloaded ``__getitem__`` functions that respond to
queries. This allows their instances to be accessed like simple dictionaries,
but the key passed to ``[]`` is converted to a query of immediate child
instances instead of a simple lookup.
"""
import operator
import re
import six
import sys
from collections import defaultdict
from itertools import chain, count
from insights.parsr.query.boolean import All, Any, Boolean, Not, pred, pred2  # noqa

# intern was a builtin until it moved to sys in python3
try:
    intern = sys.intern
except:
    pass


def _make_str(s):
    """
    Inspired by six.ensure_str in six version 1.15. See the six module for
    copyright notice.
    """
    if six.PY2 and isinstance(s, six.text_type):
        return s.encode("utf-8", "strict")
    if six.PY3 and isinstance(s, six.binary_type):
        return s.decode("utf-8", "strict")
    return s


# we need this when generating crumbs
try:
    isidentifier = str.isidentifier
except:
    isidentifier = re.compile(r"^[^\d\W]\w*\Z").match


try:
    from collections import Counter
except ImportError:
    from operator import itemgetter

    class Counter(object):
        def __init__(self, data):
            self.data = defaultdict(int)
            self.update(data)

        def update(self, data):
            for d in data:
                self.data[d] += 1

        def most_common(self, top=None):
            res = sorted(self.data.items(), key=itemgetter(1), reverse=True)
            return res[:top] if top is not None else res

ANY = None


class Entry(object):
    """
    Entry is the base class for the data model, which is a tree of Entry
    instances. Each instance has a name, attributes, a parent, and children.
    """
    __slots__ = ("_name", "attrs", "children", "parent", "lineno", "src")

    def __init__(self, name=None, attrs=None, children=None, lineno=None, src=None, set_parents=True):
        if type(name) is str:
            self._name = intern(name)
        elif isinstance(name, (six.text_type, six.binary_type)):
            self._name = intern(_make_str(name))
        else:
            self._name = name

        self.attrs = attrs if isinstance(attrs, (list, tuple)) else tuple()
        self.children = children if isinstance(children, (list, tuple)) else []
        self.parent = None
        self.lineno = lineno
        self.src = src  # insights.core.Parser instance
        if set_parents:
            for c in self.children:
                c.parent = self
        super(Entry, self).__init__()

    def __getattr__(self, name):
        """
        Allows queries based on attribute access so long as they don't conflict
        with members of the Entry class itself.
        """
        if name == "name" and self._name is not None:
            return self._name

        res = self[name]
        if res:
            return res

        # fall through for stuff like file_path, etc.
        if hasattr(self.src, name):
            return getattr(self.src, name)

        return res

    def get_keys(self):
        """
        Returns the unique names of all the children as a list.
        """
        return sorted(set(c._name for c in self.children if c._name))

    keys = get_keys

    def __dir__(self):
        """
        Exists for ipython autocompletion.
        """
        return self.get_keys() + object.__dir__(self)

    @property
    def source(self):
        p = self
        while p is not None and p.src is None:
            p = p.parent

        if p is not None and p.src is not None:
            return getattr(p.src, "file_path", p.src)

    def _crumbs_up(self):
        res = []
        segments = []
        cur = self
        while cur is not None:
            name = cur._name
            if name is not None:
                segments.append(name)
            cur = cur.parent

        if len(segments) == 0:
            res.append("")
        elif len(segments) == 1:
            res.append(segments[0])
        else:
            segments = list(reversed(segments))
            path = [segments[0]]
            for r in segments[1:]:
                r = "." + r if isidentifier(r) else '["{n}"]'.format(n=r)
                path.append(r)
            res.append("".join(path))
        return res

    def _crumbs_down(self):
        res = set()

        def inner(node, base):
            if node.children:
                for c in node.children:
                    name = str(c._name) or ""
                    if base:
                        if not isidentifier(name):
                            name = '"{n}"'.format(n=name)
                            path = base + "[" + name + "]"
                        else:
                            path = base + "." + name
                    else:
                        if not isidentifier(name):
                            name = '"{n}"'.format(n=name)
                            path = "[" + name + "]"
                        else:
                            path = name
                    inner(c, path)
            else:
                res.add(base)

        inner(self, "")
        return sorted(res)

    def get_crumbs(self, down=False):
        """
        Get the unique paths from the current entry up to the root or down
        to all of the leaves.
        """
        return self._crumbs_up() if not down else self._crumbs_down()

    crumbs = get_crumbs

    @property
    def line(self):
        """
        Returns the original first line of text that generated the ``Entry``.
        ``None`` if the model wasn't generated by an insights parser.
        """
        if self.src is not None and self.lineno is not None:
            return self.src.content[self.lineno - 1]

    @property
    def string_value(self):
        """
        Returns the string representation of all attributes separated by a
        single whilespace.
        """
        t = " ".join(["%s"] * len(self.attrs))
        return t % tuple(self.attrs)

    @property
    def value(self):
        """
        Returns ``None`` if no attributes exist, the first attribute if only
        one exists, or the ``string_value`` if more than one exists.
        """
        if len(self.attrs) == 1:
            return self.attrs[0]
        return self.string_value if len(self.attrs) > 1 else None

    @property
    def root(self):
        """
        Returns the furthest ancestor ``Entry``. If the node is already the
        furthest ancestor, ``None`` is returned.
        """
        p = self.parent
        while p is not None and p.parent is not None:
            p = p.parent
        return p

    @property
    def grandchildren(self):
        """
        Returns a flattened list of all grandchildren.
        """
        return list(chain.from_iterable(c.children for c in self.children))

    def upto(self, query):
        """
        Go up from the current node to the first node that matches query.
        """
        predicate = _desugar(query)
        parent = self.parent
        while parent is not None:
            if predicate(parent):
                return parent
            parent = parent.parent

    def select(self, *queries, **kwargs):
        """
        select uses :py:func:`compile_queries` to compile ``queries`` into a
        query function and then passes the function, the current ``Entry``
        instances children, and ``kwargs`` on to :py:func:`select`.
        """
        query = compile_queries(*queries)
        return select(query, self.children, **kwargs)

    def find(self, *queries, **kwargs):
        """
        Finds matching results anywhere in the configuration. The arguments are
        the same as those accepted by :py:func:`compile_queries`, and it
        accepts a keyword called ``roots`` that will return the ultimate root
        nodes of any results.
        """
        roots = kwargs.get("roots", False)
        return self.select(*queries, deep=True, roots=roots)

    def where(self, name, value=None):
        """
        Selects current nodes based on name and value queries of child nodes.
        If any immediate children match the queries, the parent is included in
        the results. The :py:func:``make_child_query`` function can be used to
        construct queries that act on the children as a whole instead of one
        at a time.

        Example:
        >>> from insights.parsr.query import make_child_query as q
        >>> from insights.parsr.query import from_dict
        >>> r = from_dict(load_config())
        >>> r = conf.status.conditions.where(q("status", "False") | q("type", "Progressing"))
        >>> r.message
        >>> r = conf.status.conditions.where(q("status", "False") | q("type", "Progressing"))
        >>> r.message
        >>> r.lastTransitionTime.values
        ['2019-08-04T23:17:08Z', '2019-08-04T23:32:14Z']
        """
        if isinstance(name, _EntryQuery):
            query = name.to_pyfunc()
        elif isinstance(name, Boolean):
            query = child_query(name, value).to_pyfunc()
        elif callable(name):
            def predicate(e):
                try:
                    return name(e)
                except:
                    return False
            query = predicate
        else:
            query = child_query(name, value).to_pyfunc()
        return Result(children=self.children if query(self) else [])

    def choose(self, chooser):
        """
        Run a selector function on each node. It should return a tuple, each
        element of which is some query using the node. This lets you select
        parts of each tree in a result.

        If you want to rename a field, make the element a dictionary whose
        key is the name you want and whose value is the query.

        If you want to get all the children of a particular node instead of
        specifying them individually, use the * operator in python 3.5+ or
        return the query with ".grandchildren" appended otherwise.

        Example:
        >>> from insights.parsr.query import make_child_query as q
        >>> from insights.parsr.query import from_dict

        >>> conf = from_dict(load_config())
        >>> p = (q("restartCount", gt(2)) & q("ready", False))

        >>> # get the name, the restartCount renamed to restart, the podIP
        >>> # from the node's parent, and all of the children from
        >>> # n.lastState.terminated.

        >>> # for python 3.5+
        >>> sel = lambda n: (n["name"], {"restart": n.restartCount}, n.parent.podIP, *n.lastState.terminated)

        >>> # for python 2
        >>> sel = lambda n: (n["name"], {"restart": n.restartCount}, n.parent.podIP, n.lastState.terminated.grandchildren)

        >>> conf.find(ANY).where(p).choose(sel)
        """
        results = []
        for c in self.children:
            res = chooser(c)
            res = res if isinstance(res, (list, tuple)) else (res,)
            tmp = []
            for r in res:
                if isinstance(r, dict):
                    for k, v in r.items():
                        if isinstance(v, list):
                            tmp.append(Entry(k, children=v, set_parents=False))
                        elif isinstance(v, Entry):
                            for i in v.children:
                                tmp.append(Entry(k, i.attrs, i.children, set_parents=False))
                        else:
                            tmp.append(Entry(k, attrs=(v,), set_parents=False))
                else:
                    if isinstance(r, list):
                        tmp.extend(r)
                    else:
                        if isinstance(r, _Choose):
                            tmp.extend(r.grandchildren)
                        else:
                            tmp.extend(r.children)
            if tmp:
                results.append(Entry(children=tmp, set_parents=False))
        return _Choose(results)

    @property
    def section(self):
        return None

    @property
    def section_name(self):
        return None

    @property
    def sections(self):
        """
        Returns all immediate children that are instances of
        :py:class:`Section`.
        """
        return Result(children=[c for c in self.children if isinstance(c, Section)])

    @property
    def directives(self):
        """
        Returns all immediate children that are instances of
        :py:class:`Directive`.
        """
        return Result(children=[c for c in self.children if isinstance(c, Directive)])

    def __contains__(self, key):
        return len(self[key]) > 0

    def __len__(self):
        return len(self.children)

    def __getitem__(self, query):
        if isinstance(query, (int, slice)):
            return self.children[query]
        query = _desugar(query)
        return Result(children=[c for c in self.children if query(c)])

    def __bool__(self):
        return bool(self._name or self.attrs or self.children)

    def __repr__(self):
        return "\n".join(pretty_format(self))

    __nonzero__ = __bool__


class Section(Entry):
    """
    A Section is an ``Entry`` composed of other Sections and
    :py:class:`Directive` instances.
    """
    @property
    def section(self):
        """
        Returns the name of the section.
        """
        return self.name

    @property
    def section_name(self):
        """
        Returns the value of the section.
        """
        return self.value


class Directive(Entry):
    """
    A Directive is an ``Entry`` that represents a single option or named value.
    They are normally found in :py:class:`Section` instances.
    """
    @property
    def section(self):
        if self.parent:
            return self.parent.section

    @property
    def section_name(self):
        if self.parent:
            return self.parent.section_name


class Result(Entry):
    """
    Result is an Entry whose children are the results of a query.
    """
    def __init__(self, children=None):
        super(Result, self).__init__()
        self.children = children or []

    def get_keys(self):
        """
        Returns the unique names of all the grandchildren as a list.
        """
        return sorted(set(c.name for c in self.grandchildren))

    keys = get_keys

    def get_crumbs(self, down=False):
        """
        Get the unique names from the current locations to the roots.
        """
        res = chain.from_iterable(c.get_crumbs(down=down) for c in self.children)
        return sorted(set(res))

    crumbs = get_crumbs

    @property
    def sources(self):
        res = []
        for c in self.children:
            src = c.source
            if src is not None:
                res.append(src)
        return sorted(set(res))

    @property
    def line(self):
        """
        Returns the line of the child if only one child exists. This helps
        queries behave more like dictionaries when you know only one result
        should exist.
        """
        if len(self.children) == 0:
            return None

        if len(self.children) == 1:
            return self.children[0].line

        raise Exception("More than one value to return.")

    @property
    def string_value(self):
        """
        Returns the string value of the child if only one child exists. This
        helps queries behave more like dictionaries when you know only one
        result should exist.
        """
        if len(self.children) == 0:
            return None

        if len(self.children) == 1:
            return self.children[0].string_value

        raise Exception("More than one value to return.")

    @property
    def value(self):
        """
        Returns the value of the child if only one child exists. This helps
        queries behave more like dictionaries when you know only one result
        should exist.
        """
        if len(self.children) == 0:
            return None

        if len(self.children) == 1:
            return self.children[0].value

        raise Exception("More than one value to return.")

    @property
    def parents(self):
        """
        Returns all of the deduplicated parents as a list. If a child has no
        parent, the child itself is treated as the parent.
        """
        parents = []
        seen = set()
        for c in self.children:
            p = c.parent
            if p is not None:
                if p not in seen:
                    parents.append(p)
                    seen.add(p)
            elif c not in seen:
                parents.append(c)
                seen.add(c)
        return Result(children=parents)

    @property
    def roots(self):
        """
        Returns the furthest ancestor ``Entry`` instances of all children. If a
        child has no furthest ancestor, the child itself is treated as a root.
        """
        roots = []
        seen = set()
        for c in self.children:
            r = c.root
            if r is not None:
                if r not in seen:
                    roots.append(r)
                    seen.add(r)
            elif c not in seen:
                roots.append(c)
                seen.add(c)
        return Result(children=roots)

    @property
    def values(self):
        """
        Returns the values of all the children as a list.
        """
        return [c.value for c in self.children if c.value is not None]

    @property
    def unique_values(self):
        """
        Returns the unique values of all the children as a list.
        """
        return sorted(set(c.value for c in self.children if c.value is not None))

    def upto(self, query):
        """
        Go up from the current results to the first nodes that match query.
        """
        roots = []
        seen = set()
        for c in self.children:
            root = c.upto(query)
            if root is not None and root not in seen:
                roots.append(root)
                seen.add(root)
        return Result(children=roots)

    def nth(self, n):
        """
        If the results are from a list beneath a node, get the nth element of
        the results for each unique parent.

        Example:
        ``conf.status.conditions.nth(0)`` will get the 0th condition of each
        status.
        """
        tmp = defaultdict(list)
        for c in self.children:
            tmp[c.parent].append(c)

        results = []
        for _, v in tmp.items():
            try:
                r = v[n]
                if isinstance(r, list):
                    results.extend(r)
                else:
                    results.append(v[n])
            except:
                pass
        return Result(children=results)

    def select(self, *queries, **kwargs):
        query = compile_queries(*queries)
        return select(query, self.grandchildren, **kwargs)

    def where(self, name, value=None):
        """
        Selects current nodes based on name and value queries of child nodes.
        If any immediate children match the queries, the parent is included in
        the results. The :py:func:``make_child_query`` function can be used to
        construct queries that act on the children as a whole instead of one
        at a time.

        Example:
        >>> from insights.parsr.query import make_child_query as q
        >>> from insights.parsr.query import from_dict
        >>> r = from_dict(load_config())
        >>> r = conf.status.conditions.where(q("status", "False") | q("type", "Progressing"))
        >>> r.message
        >>> r = conf.status.conditions.where(q("status", "False") | q("type", "Progressing"))
        >>> r.message
        >>> r.lastTransitionTime.values
        ['2019-08-04T23:17:08Z', '2019-08-04T23:32:14Z']
        """
        if isinstance(name, _EntryQuery):
            query = name.to_pyfunc()
        elif isinstance(name, Boolean):
            query = child_query(name, value).to_pyfunc()
        elif callable(name):
            def predicate(e):
                try:
                    return name(e)
                except:
                    return False
            query = predicate
        else:
            query = child_query(name, value).to_pyfunc()
        results = []
        seen = set()
        for c in self.children:
            if c not in seen and query(c):
                results.append(c)
        return Result(children=results)

    def to_df(self):
        import pandas as pd

        res = []
        for p in self.children:
            try:
                d = dict((c._name, c.attrs[0]) for c in p.children if len(c.attrs) == 1)
                res.append(d)
            except:
                pass
        return pd.DataFrame(res)

    def most_common(self, top=None):
        """
        Returns the distribution of values returned by queries that return a
        single value for each node.
        """
        res = [c.attrs[0] for c in self.children if len(c.attrs) == 1]
        return Counter(res).most_common(top)

    def __getitem__(self, query):
        if isinstance(query, (int, slice)):
            return self.children[query]
        query = _desugar(query)
        return Result(children=[c for c in self.grandchildren if query(c)])


class _Choose(Result):
    """
    Marker class that allows us to detected nested calls to choose
    """
    pass


class _EntryQuery(object):
    """
    _EntryQuery is the base class of all other query classes.
    """
    def __and__(self, other):
        return _AllEntryQuery(self, other)

    def __or__(self, other):
        return _AnyEntryQuery(self, other)

    def __invert__(self):
        return _NotEntryQuery(self)

    def to_pyfunc(self):
        ver = sys.version_info
        if ver[0] == 2 and ver[1] == 6:
            return self.test

        env = {}
        ids = count()

        def expr(b):
            if isinstance(b, _AllEntryQuery):
                return "(" + " and ".join(expr(p) for p in b.exprs) + ")"
            elif isinstance(b, _AnyEntryQuery):
                return "(" + " or ".join(expr(p) for p in b.exprs) + ")"
            elif isinstance(b, _NotEntryQuery):
                return "(" + "not " + expr(b.query) + ")"
            else:
                num = next(ids)
                func = "func_{num}".format(num=num)
                env[func] = b.test
                return func + "(value)"

        func = """
def predicate(value):
    try:
        return {body}
    except Exception as ex:
        return False
        """.format(body=expr(self))

        six.exec_(func, env, env)
        return env["predicate"]


class _AllEntryQuery(_EntryQuery, All):
    pass


class _AnyEntryQuery(_EntryQuery, Any):
    pass


class _NotEntryQuery(_EntryQuery, Not):
    pass


class _AllAttrQuery(_EntryQuery):
    def __init__(self, expr):
        self.expr = expr

    def test(self, e):
        return all(self.expr(a) for a in e.attrs)


class _AnyAttrQuery(_EntryQuery):
    def __init__(self, expr):
        self.expr = expr

    def test(self, e):
        return any(self.expr(a) for a in e.attrs)


def all_(expr):
    """
    Use to express that ``expr`` must succeed on all attributes for the query
    to be successful. Only works against :py:class:`Entry` attributes.
    """
    return _AllAttrQuery(_desugar_attr(expr))


def any_(expr):
    """
    Use to express that ``expr`` can succeed on any attribute for the query to be
    successful. Only works against :py:class:`Entry` attributes.
    """
    return _AnyAttrQuery(_desugar_attr(expr))


class ChildQuery(_EntryQuery):
    """
    Returns True if any child entry passes the query.
    """
    def __init__(self, expr):
        self.expr = expr

    def test(self, e):
        return any(self.expr(n) for n in e.children)


def child_query(name, value=None):
    """
    Converts a query into a ChildQuery that works on all child entries at once
    to determine if the current entry is accepted.
    """
    q = _desugar((name, value) if value is not None else name)
    return ChildQuery(q)


make_child_query = child_query


def _desugar_name(q):
    if q is None:
        return lambda _: True
    if isinstance(q, Boolean):
        f = q.to_pyfunc()
        return lambda e: f(e._name)
    if callable(q):
        def predicate(e):
            try:
                return q(e._name)
            except:
                return False
        return predicate
    return lambda e: e._name == q


def _desugar_attr(q):
    if isinstance(q, Boolean):
        return q.to_pyfunc()
    if callable(q):
        def predicate(v):
            try:
                return q(v)
            except:
                return False
        return predicate
    return lambda v: v == q


def _desugar_attrs(q):
    if not q:
        return
    if len(q) == 1:
        q = q[0]
        return q if isinstance(q, _EntryQuery) else _AnyAttrQuery(_desugar_attr(q))
    else:
        # conf[name, q0, q1] means "name and (q0 or q1 for any attribute)"
        attr_queries = [_desugar_attr(a) for a in q]
        return _AnyAttrQuery(lambda v: any(p(v) for p in attr_queries))


def _desugar(q):
    if isinstance(q, _EntryQuery):
        return q.to_pyfunc()
    if isinstance(q, tuple):
        name_query = _desugar_name(q[0])
        attrs_query = _desugar_attrs(q[1:])
        if attrs_query:
            aq = attrs_query.to_pyfunc()
            return lambda e: name_query(e) and aq(e)
        return name_query
    if q is None:
        return lambda _: True
    return _desugar_name(q)


def _flatten(nodes):
    """
    Flatten the config tree into a list of nodes.
    """
    def inner(n):
        yield n
        for i in chain.from_iterable(inner(c) for c in n.children):
            yield i
    return list(chain.from_iterable(inner(n) for n in nodes))


def compile_queries(*queries):
    """
    compile_queries returns a function that will execute a list of query
    expressions against an :py:class:`Entry`. The first query is run against
    the current entry's children, the second query is run against the children
    of the children remaining from the first query, and so on.

    If a query is a single object, it matches against the name of an Entry. If
    it's a tuple, the first element matches against the name, and subsequent
    elements are tried against each individual attribute. The attribute results
    are `or'd` together and that result is `anded` with the name query. Any
    query that raises an exception is treated as ``False``.
    """
    queries = [_desugar(q) for q in queries]

    def match(qs, nodes):
        q = qs[0]
        qs = qs[1:]
        res = [n for n in nodes if q(n)]
        if qs and res:
            gc = list(chain.from_iterable(n.children for n in res))
            return match(qs, gc)
        return res

    def inner(nodes):
        return Result(children=match(queries, nodes))
    return inner


def select(query, nodes, deep=False, roots=False):
    """
    select runs query, a function returned by :py:func:`compile_queries`,
    against a list of :py:class:`Entry` instances. If you pass ``deep=True``,
    select recursively walks each entry in the list and accumulates the
    results of running the query against it. If you pass ``roots=True``,
    select returns the deduplicated set of final ancestors of all successful
    queries. Otherwise, it returns the matching entries.
    """
    results = query(_flatten(nodes)) if deep else query(nodes)

    if not roots:
        return Result(children=results)

    seen = set()
    top = []
    for r in results:
        root = r.root
        if root not in seen:
            seen.add(root)
            top.append(root)
    return Result(children=top)


def from_dict(orig, src=None):
    """
    from_dict is a helper function that does its best to convert a python dict
    into a tree of :py:class:`Entry` instances that can be queried.
    """
    def inner(d):
        result = []
        for k, v in d.items():
            if isinstance(v, dict):
                result.append(Entry(name=k, children=inner(v)))
            elif isinstance(v, list):
                res = [Entry(name=k, children=inner(i)) if isinstance(i, dict) else i for i in v]
                if res:
                    if isinstance(res[0], Entry):
                        result.extend(res)
                    else:
                        result.append(Entry(name=k, attrs=tuple(res)))
                else:
                    result.append(Entry(name=k))
            else:
                result.append(Entry(name=k, attrs=(v,)))
        return tuple(result)
    return Entry(children=inner(orig), src=src)


def pretty_format(root, indent=4):
    """
    pretty_format generates a text representation of a model as a list of
    lines.
    """
    results = []

    def sep():
        if results and results[-1] != "":
            results.append("")

    def inner(d, prefix=""):
        if isinstance(d, Result):
            for c in d.children:
                inner(c, prefix)
            return
        if d.children:
            sep()
            name = d._name or ""
            header = name if not d.attrs else " ".join([name, d.string_value])
            if header:
                results.append(prefix + "[" + header + "]")
            prep = prefix + (" " * indent)
            for c in d.children:
                inner(c, prep)
            sep()
        else:
            results.append(prefix + (d._name or "") + ": " + d.string_value)

    inner(root)
    return results


# These predicates can be used in queries.
lt = pred2(operator.lt)
le = pred2(operator.le)
eq = pred2(operator.eq)
gt = pred2(operator.gt)
ge = pred2(operator.ge)

isin = pred2(lambda v, values: v in set(values))
matches = pred2(lambda v, pat: re.search(pat, v))

contains = pred2(operator.contains)
startswith = pred2(str.startswith)
endswith = pred2(str.endswith)

ieq = pred2(operator.eq, ignore_case=True)
icontains = pred2(operator.contains, ignore_case=True)
istartswith = pred2(str.startswith, ignore_case=True)
iendswith = pred2(str.endswith, ignore_case=True)

first = 0
last = -1



import doctest

from insights.parsers import vmware_tools_conf
from insights.tests import context_wrap

CONF = '''
[guestinfo]
disable-query-diskinfo = true

[logging]
log = true

vmtoolsd.level = debug
vmtoolsd.handler = file
vmtoolsd.data = /tmp/vmtoolsd.log
'''


def test_vmware_tools_conf():
    conf = vmware_tools_conf.VMwareToolsConf(context_wrap(CONF))
    assert list(conf.sections()) == ['guestinfo', 'logging']
    assert conf.has_option('guestinfo', 'disable-query-diskinfo') is True
    assert conf.getboolean('guestinfo', 'disable-query-diskinfo') is True
    assert conf.get('guestinfo', 'disable-query-diskinfo') == 'true'
    assert conf.get('logging', 'vmtoolsd.handler') == 'file'
    assert conf.get('logging', 'vmtoolsd.data') == '/tmp/vmtoolsd.log'


def test_vmware_tools_conf_documentation():
    failed_count, tests = doctest.testmod(
        vmware_tools_conf,
        globs={'conf': vmware_tools_conf.VMwareToolsConf(context_wrap(CONF))}
    )
    assert failed_count == 0



"""
WIP: Will require tweaks as additional Oracle functionality is added.

Curently only gathers information from a database using generic init.ora file.
"""

import re
from string import whitespace
from .. import Parser, parser, get_active_lines
from insights.specs import Specs

# spfiles tend to contain strings of control characters usually bounded by 'C"'
# and always ending with an ASCII NULL. This regex describes those strings for
# removal
cleanup = re.compile('(C")?[\01\00].*\00')


def _parse_oracle(lines):
    """
    Performs the actual file parsing, returning a dict of the config values
    in a given Oracle DB config file.

    Despite their differences, the two filetypes are similar enough to
    allow idential parsing.
    """
    config = {}

    for line in get_active_lines(lines):
        # Check for NULL in line to begin control char removal
        if '\00' in line:
            line = cleanup.sub('', line)
        if '=' in line:
            (key, value) = line.split('=', 1)
            key = key.strip(whitespace + '"\'').lower()
            if ',' in line:
                value = [s.strip(whitespace + '"\'').lower() for s in value.split(',')]
            else:
                value = value.strip(whitespace + '"\'').lower()
            config[key] = value

    return config


@parser(Specs.init_ora)
class OraclePfile(Parser):
    """
    Parse Oracle database settings contained in an init.ora file.

    Returns a dict containing the name and pertinent settings of the database.

    Future iterations may have to check multiple .ora files for multiple DBs.
    """

    def parse_content(self, content):
        self.data = dict(_parse_oracle(content))


@parser(Specs.spfile_ora)
class OracleSpfile(Parser):
    """
    Parse Oracle database settings contained in an spfile.

    Returns a dict containing the name and pertinent settings of the database.
    """

    def parse_content(self, content):
        self.data = dict(_parse_oracle(content))



from insights.core.plugins import rule, make_fail, make_pass
from insights.parsers import installed_rpms, uname as uname_mod


@rule(optional=[installed_rpms.InstalledRpms, uname_mod.Uname])
def report(rpms, uname):
    if rpms is not None:
        bash_ver = rpms.get_max('bash')
        if uname is not None:
            return make_pass('PASS', bash_ver=bash_ver.nvr, uname_ver=uname.version)
        else:
            return make_fail('FAIL', bash_ver=bash_ver.nvr, path=rpms.file_path)

    # implicit return None



"""
``/sys/bus/vmbus/`` VMBus info
==============================

SysVmbusDeviceID - file ``/sys/bus/vmbus/devices/*/device_id``
--------------------------------------------------------------

SysVmbusClassID - file ``/sys/bus/vmbus/devices/*/class_id``
------------------------------------------------------------
"""

from insights import parser, Parser
from insights.parsers import SkipException
from insights.specs import Specs

# Please refer to
# https://github.com/torvalds/linux/blob/master/tools/hv/lsvmbus#L23 for
# full list of Class ID mapping.
VMBUS_DEV_DICT = {
    '44c4f61d-4444-4400-9d52-802e27ede19f': 'PCI Express pass-through',
    'da0a7802-e377-4aac-8e77-0558eb1073f8': 'Synthetic framebuffer adapter'
}


@parser(Specs.sys_vmbus_device_id)
class SysVmbusDeviceID(Parser):
    """Parse the file ``/sys/bus/vmbus/devices/*/device_id``

    Sample content::

        {47505500-0001-0000-3130-444531444234}

    Raises:
        SkipException: When nothing need to parse.

    Attributes:
        id(str): Device ID

    Examples::

        >>> vmbus_device.id
        '47505500-0001-0000-3130-444531444234'
    """
    def parse_content(self, content):
        if not content or len(content) != 1:
            raise SkipException()
        self.id = content[0].strip('{}\n')


@parser(Specs.sys_vmbus_class_id)
class SysVmbusClassID(Parser):
    """Parse the file ``/sys/bus/vmbus/devices/*/class_id``

    Sample content::

        {44c4f61d-4444-4400-9d52-802e27ede19f}

    Raises:
        SkipException: When nothing need to parse.

    Attributes:
        id(str): Class ID
        desc(str): Description

    Examples::

        >>> vmbus_class.id
        '44c4f61d-4444-4400-9d52-802e27ede19f'
        >>> vmbus_class.desc
        'PCI Express pass-through'
    """
    def parse_content(self, content):
        if not content or len(content) != 1:
            raise SkipException()
        self.id = content[0].strip('{}\n')
        self.desc = VMBUS_DEV_DICT.get(self.id, 'Unknown')



import doctest
from insights.parsers import rhosp_release
from insights.tests import context_wrap


ROCKY = """
Red Hat OpenStack Platform release 14.0.0 RC (Rocky)
""".strip()


PIKE = """
Red Hat OpenStack Platform release 12.0 Beta (Pike)
""".strip()


def test_rhosp_release():
    rocky = rhosp_release.RhospRelease(context_wrap(ROCKY))
    assert rocky.product == "Red Hat OpenStack Platform"
    assert rocky.version == '14.0.0'
    assert rocky.code_name == "Rocky"

    pike = rhosp_release.RhospRelease(context_wrap(PIKE))
    assert pike.product == "Red Hat OpenStack Platform"
    assert pike.version == '12.0'
    assert pike.code_name == "Pike"


def test_documentation():
    failed_count, tests = doctest.testmod(
        rhosp_release,
        globs={'release': rhosp_release.RhospRelease(context_wrap(ROCKY))})
    assert failed_count == 0



from insights.parsers import smbstatus
from insights.parsers.smbstatus import SmbstatusS, Smbstatusp, Statuslist
from insights.tests import context_wrap
from insights.parsers import ParseException, SkipException
import pytest
import doctest

SMBSTATUSS = """

Service      pid     Machine       Connected at                     Encryption   Signing
---------------------------------------------------------------------------------------------
share_test1  12668   10.66.208.149 Wed Sep 27 10:33:55 AM 2017 CST  -            -
share_test2  12648   10.66.208.159 Wed Sep 27 11:33:55 AM 2017 CST  -            -
share_test3  13628   10.66.208.169 Wed Sep 27 12:33:55 AM 2017 CST  -            -

"""

SMBSTATUSP = """

Samba version 4.6.2
PID     Username     Group        Machine                                   Protocol Version  Encryption           Signing
----------------------------------------------------------------------------------------------------------------------------------------
12668   testsmb       zjjsmb       10.66.208.149 (ipv4:10.66.208.149:44376)  SMB2_02           -                    -
12648   test2smb      test2smb     10.66.208.159 (ipv4:10.66.208.159:24376)  SMB2_02           -                    -
13628   test3smb      test3smb     10.66.208.169 (ipv4:10.66.208.169:34376)  SMB2_02           -                    -
"""

SMBSTATUSS_EXP = """

xService      pid     Machine       Connected at                     Encryption   Signing
---------------------------------------------------------------------------------------------
share_test1  12668   10.66.208.149 Wed Sep 27 10:33:55 AM 2017 CST  -            -
share_test2  12648   10.66.208.159 Wed Sep 27 11:33:55 AM 2017 CST  -            -
share_test3  13628   10.66.208.169 Wed Sep 27 12:33:55 AM 2017 CST  -            -

"""

SMBSTATUSP_EXP1 = """
Can't open sessionid.tdb
"""

SMBSTATUSP_EXP2 = """
"""

SMBSTATUSP_DOC = """
Samba version 4.6.2
PID     Username     Group        Machine                                   Protocol Version  Encryption           Signing
--------------------------------------------------------------------------------------------------------------------------
12668   testsmb       testsmb       10.66.208.149 (ipv4:10.66.208.149:44376)  SMB2_02           -                    -
"""

SMBSTATUSS_DOC = """
Service      pid     Machine       Connected at                     Encryption   Signing
----------------------------------------------------------------------------------------
share_test   13668   10.66.208.149 Wed Sep 27 10:33:55 AM 2017 CST  -            -
"""


def test_smbstatuss():
    smbstatuss = SmbstatusS(context_wrap(SMBSTATUSS))
    assert smbstatuss.data[2]["pid"] == '13628'
    assert smbstatuss.data[1]["Connected_at"] == 'Wed Sep 27 11:33:55 AM 2017 CST'
    assert smbstatuss.data[1]["Encryption"] == '-'
    for result in smbstatuss:
        if result["Service"] == "share_test1":
            assert result["pid"] == "12668"


def test_smbstatusp():
    smbstatusp = Smbstatusp(context_wrap(SMBSTATUSP))
    assert smbstatusp.data[2]["Username"] == 'test3smb'
    assert smbstatusp.data[1]["Protocol_Version"] == 'SMB2_02'
    assert smbstatusp.data[1]["Signing"] == '-'
    for result in smbstatusp:
        if result["PID"] == "12668":
            assert result["Username"] == "testsmb"


def test_statuslist_empty_exp():
    with pytest.raises(SkipException) as pe:
        Statuslist(context_wrap(''))
        assert "Empty content." in str(pe)


def test_statuslist_exp():
    with pytest.raises(ParseException) as pe:
        Statuslist(context_wrap("Can't open sessionid.tdb"))
        assert "Can't open sessionid.tdb" in str(pe)


def test_smbstatusS_exp():
    with pytest.raises(ParseException) as pe:
        SmbstatusS(context_wrap(SMBSTATUSS_EXP))
        assert "Cannot find the header line." in str(pe)


def test_smbstatusp_exp():
    with pytest.raises(ParseException) as pe:
        Smbstatusp(context_wrap(SMBSTATUSP_EXP1))
        assert "Cannot find the header line." in str(pe)

    with pytest.raises(SkipException) as pe:
        Smbstatusp(context_wrap(SMBSTATUSP_EXP2))
        assert "Empty content." in str(pe)


def test_smbstatus_doc():
    env = {
        'SmbstatusS': SmbstatusS,
        'smbstatuss_info': SmbstatusS(context_wrap(SMBSTATUSS_DOC)),
        'Smbstatusp': Smbstatusp,
        'smbstatusp_info': Smbstatusp(context_wrap(SMBSTATUSP_DOC))
    }
    failed, total = doctest.testmod(smbstatus, globs=env)
    assert failed == 0



import os
import sys
from insights.client.constants import InsightsConstants as constants
from insights.client.apps.ansible.playbook_verifier import verify, loadPlaybookYaml, PlaybookVerificationError

skipVerify = False


def read_playbook():
    """
    Read in the stringified playbook yaml from stdin
    """
    unverified_playbook = ''
    for line in sys.stdin:
        unverified_playbook += line

    return unverified_playbook


if (os.environ.get('SKIP_VERIFY')):
    skipVerify = True

try:
    playbook = read_playbook()
    playbook_yaml = loadPlaybookYaml(playbook)
    verified_playbook = verify(playbook_yaml, skipVerify)
except PlaybookVerificationError as err:
    sys.stderr.write(err.message)
    sys.exit(constants.sig_kill_bad)

print(playbook)
